;  ----------------------------LICENSE NOTICE-----------------------------
;
;  This file is part of the Hey Sailor! game for Amstrad CPC 464.
;  Copyright (C) 2019  Daniel Bravo García  (dani_jijona@hotmail.com)
;  Copyright (C) 2019  David Navas Díaz   (dnavasdiaz97@gmail.com)
;  Copyright (C) 2019  Juan Manuel Ruiz Culiañez  (pokejuanma@gmail.com)
;
;  Hey Sailor! is free software: you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  Hey Sailor! is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with Hey Sailor!. If not, see <http://www.gnu.org/licenses/>.
;
; ----------------------------------------------------------------------



;;
;;	ENTITY MANAGER
;;

.include "entity_manager.h.s"
.include "cpct_functions.h.s"
.include "assets/assets.h.s"
.include "cpctelera.h.s"
.include "sys/render_system.h.s"
.include "sys/input_system.h.s"

_num_entities:: .db 0
_last_elem_ptr:: .dw  _entity_array 


DefineEntityArray _entity_array, max_entities

;;Declaraciones para el array de enemigos
max_enemies == 5
_num_enemies:: .db 0
_last_enemy_ptr:: .dw  _enemy_array 
DefineEnemyArray _enemy_array, max_enemies

;; PARA EL ARRAY DE BALAS
max_bullets == 10
_num_bullets:: .db 0
_last_bullet_ptr:: .dw  _bullet_array 
DefineBulletArray _bullet_array, max_bullets

;; PARA EL ARRAY DE LOS FUERTES
max_forts == 5
_num_forts:: .db 0
_last_fort_ptr:: .dw  _fort_array 
DefineFortArray _fort_array, max_forts


;; DEFINE ENTITIES
                  ;;    x,  y, vx, vy, w   h, hp,       type,     ai type,            sprite,       
DefineEntity  player,  37,  150, 0, 0,  4, 16, 3,  e_type_player, e_ai_type_no_ai, _player2_sp_00
;DefineEntity  enemy,   20, 100, 1, 1,  4, 16, 1,  e_type_enemy,  _enemy_sp_00
;DefineEntity  enemy2,  40,  80, 1, 1,  4, 16, 1,  e_type_enemy,  _enemy_sp_00
;;PRUEBA
;DefineEntity  bullet,  50, 100, 0, 1,  2, 8, 1,  e_type_bullet, _bullet_sp
;DefineEntity  bullet1, 50, 150, 1, 1,  2, 8, 1,  e_type_bullet, _bullet_sp
DefineEntity  fort,    50, 50  , 0, 0,  8, 32, 5, e_type_fort  ,  e_ai_type_spawn_fort, _base_enemy_sp_0

;; PROCEDURALES
DefineEntity  bull,     0,   0, 0, 0,  2, 8, 1,  e_type_bullet, e_ai_type_no_ai, _bullet_sp ; sus primeros 4 ceros se reescribiran
DefineEntity  enemy_bull, 0,   0, 0, 0,  2, 8, 1,  e_type_enemy_bullet, e_ai_type_no_ai, _enemy_bullet_sp ; sus primeros 4 ceros se reescribiran
DefineEntity  enem,     0,   0, 0, 0,  4, 16, 1,  e_type_enemy,  e_ai_type_shot_enemy, _enemy_sp_00 


entityman_init::
   ;; Reset num_entities
	;; Reset entities pointer
	call reset_enemy_array
	call reset_bullet_array
	call reset_fort_array
      call reset_entity_array

      call check_player_hp_and_create
	  ;ld 	hl, #player
      ;call entityman_create

      ;ld   hl, #fort
      ;call entityman_create_fort

      ;ld 	hl, #enemy
      ;call entityman_create_enemy

      ;ld   hl, #enemy2
      ;call entityman_create_enemy

      ;ld 	hl, #bullet
      ;call entityman_create_bullet

      
ret

;; ---------------------------------------- GETTERS

;; GET DEL ARRAY DE ENEMIGOS
entityman_getEnemyArray_IX::
	ld 		ix, #_enemy_array
	ret
entityman_getNumEnemies_A::
	ld 		a, (_num_enemies)
	ret
;; PARA PRUEBAS
entityman_getEnemyArray_HL::
	ld 		hl, #_enemy_array
	ret


;; GET DEL ARRAY DE BALAS
entityman_getBulletArray_IX::
	ld 		ix, #_bullet_array
	ret
entityman_getNumBullets_A::
	ld 		a, (_num_bullets)
	ret
entityman_getBulletArray_HL::
	ld 		hl, #_bullet_array
	ret


;; GET DEL ARRAY DE FUERTES
entityman_getFortArray_IX::
	ld 		ix, #_fort_array
	ret
entityman_getFortArray_IY::		
	ld 		iy, #_fort_array		
	ret
entityman_getNumForts_A::
	ld 		a, (_num_forts)
	ret
entityman_getFortArray_HL::
	ld 		hl, #_fort_array
	ret


;; GET DEL ARRAY DE ENTIDADES
entityman_getEntityArray_IX::
	ld 		ix, #_entity_array
	ld    a, (_num_entities)
ret

entityman_getEntityArray_IY::
	ld 		iy, #_entity_array
ret

entityman_getNumEntities_A::
	ld 		a, (_num_entities)
	ret

;; ---------------------------------------- CREADORES

;; INPUT
;;	HL: Pointer to Entity initializer bytes
entityman_create::
	;; comprobamos si hemos llegado al maximo de entidades
	ld	iy, #_num_entities	;; variable
	ld	a, #max_entities		;; constante
	cp	a, 0(iy)
	jr	z, no_cabe

	ld 	de, (_last_elem_ptr)
	ld 	bc, #sizeof_e
	ldir

	ld 	a, (_num_entities)
	inc 	a
	ld    (_num_entities), a

	ld	hl, (_last_elem_ptr)
	ld 	bc, #sizeof_e
	add	hl, bc
	ld	(_last_elem_ptr), hl

no_cabe:

ret 	

;; CREA UN ENEMIGO Y LO METE EN EL ARRAY DE ENEMIGOS
;; INPUT
;;	HL: Pointer to enemy initializer bytes
entityman_create_enemy::
		;; comprobamos si hemos llegado al maximo de enemgos
	ld	iy, #_num_enemies	;; variable
	ld	a, #max_enemies		;; constante
	cp	a, 0(iy)
	jr	z, no_cabe_enemigo

	_pos_entity == #0000
	ld 	a, (_num_enemies)	;; incrementamos el numero de enemigos
	inc 	a
	ld    (_num_enemies), a

	ld	de, (_last_elem_ptr)	;; guardamos la posicion donde se va a escribir la nueva entidad
	push	de
	;; Ahora creamos la entidad metiendo sus datos en nuestro array de entidades
	call  entityman_create
	;; Ahora escribimos la direccion del EntityArray donde esta esta entidad

	pop 	de		;; posicion de la entidad
	ld	(_pos_entity), de
	ld 	hl, (_last_enemy_ptr)
	ld 	de, #_pos_entity	;; tenemos que escribir en la posicion de DE lo que tiene HL
	ex 	de, hl
	ld 	bc, #sizeof_ptr_to_enemy
	ldir		;; escribimos la direccion en el final de nuestro array de enemigos

	;; Actualizamos el puntero al ultimo enemigo
	ld	hl, (_last_enemy_ptr)
	ld 	bc, #sizeof_ptr_to_enemy
	add	hl, bc
	ld	(_last_enemy_ptr), hl

no_cabe_enemigo:
ret 


;; CREA UNA BALA Y LA METE EN EL ARRAY DE BALAS
;; INPUT
;;	HL: Pointer to bullet initializer bytes
entityman_create_bullet::
		;; comprobamos si hemos llegado al maximo de balas
	ld	iy, #_num_bullets	;; variable
	ld	a, #max_bullets		;; constante
	cp	a, 0(iy)
	jr	z, no_cabe_bala

	_pos_entity_bullet == #0000
	ld 	a, (_num_bullets)	;; incrementamos el numero de balas
	inc 	a
	ld    (_num_bullets), a

	ld	de, (_last_elem_ptr)	;; guardamos la posicion donde se va a escribir la nueva entidad
	push	de
	;; Ahora creamos la entidad metiendo sus datos en nuestro array de entidades
	call  entityman_create
	;; Ahora escribimos la direccion del EntityArray donde esta esta entidad

	pop 	de		;; posicion de la entidad
	ld	(_pos_entity_bullet), de
	ld 	hl, (_last_bullet_ptr)
	ld 	de, #_pos_entity_bullet	;; tenemos que escribir en la posicion de DE lo que tiene HL
	ex 	de, hl
	ld 	bc, #sizeof_ptr_to_bullet
	ldir		;; escribimos la direccion en el final de nuestro array de balas

	;; Actualizamos el puntero al ultimo enemigo
	ld	hl, (_last_bullet_ptr)
	ld 	bc, #sizeof_ptr_to_bullet
	add	hl, bc
	ld	(_last_bullet_ptr), hl


no_cabe_bala:
ret 


;; CREA UN FUERTE Y LO METE EN EL ARRAY DE FUERTES
;; INPUT
;;	HL: Pointer to fort initializer bytes
entityman_create_fort::
			;; comprobamos si hemos llegado al maximo de balas
	ld	iy, #_num_forts	;; variable
	ld	a, #max_forts		;; constante
	cp	a, 0(iy)
	jr	z, no_cabe_fuerte

	_pos_entity_fort == #0000
	ld 	a, (_num_forts)	;; incrementamos el numero de fuertes
	inc 	a
	ld    (_num_forts), a

	ld	de, (_last_elem_ptr)	;; guardamos la posicion donde se va a escribir la nueva entidad
	push	de
	;; Ahora creamos la entidad metiendo sus datos en nuestro array de entidades
	call  entityman_create
	;; Ahora escribimos la direccion del EntityArray donde esta esta entidad

	pop 	de		;; posicion de la entidad
	ld	(_pos_entity_fort), de
	ld 	hl, (_last_fort_ptr)
	ld 	de, #_pos_entity_fort	;; tenemos que escribir en la posicion de DE lo que tiene HL
	ex 	de, hl
	ld 	bc, #sizeof_ptr_to_fort
	ldir		;; escribimos la direccion en el final de nuestro array de fuertes

	;; Actualizamos el puntero al ultimo fuerte
	ld	hl, (_last_fort_ptr)
	ld 	bc, #sizeof_ptr_to_fort
	add	hl, bc
	ld	(_last_fort_ptr), hl

no_cabe_fuerte:
ret 


;; ---------------------------------------- DESTRUCTORES

;; DESTRUYE UNA ENTIDAD DEL ARRAY DE ENTIDADES
;; INPUT
;;	HL: Pointer to entity 
entityman_destroy_entity::
	;; para destruir la colision, hl debe apuntar a ella
	push hl
	ld	a, l
	ld__ixl_a
	ld	a, h
	ld__ixh_a
	call rendersys_clear_entity
	
	pop hl
	;call update_entities		;; actualizamos los arrays de punteros
	;;_uno = #0001
	;; Comprobamos si es la ultima
	push	hl		;; guardamos hl
	ld 	de, (_last_elem_ptr)
	ld 	bc, #sizeof_e
	add	hl, bc		;;sumamos tamano
	sbc	hl, de	;; si da 0, era la ultima
	pop	hl	;; hl = Pointer to entity 
	jr	z, _last_entity

_NOT_last_entity:
	;; tenemos que mover la ultima entidad a la posicion de hl

	;; primero calculamos la posicion de la ultima entidad
	ex	de, hl	;; de tenia _last_elem_ptr
				;; ahora DE tiene la entidad a borrar
	sbc	hl, bc	;; hl = la ultima entidad
				;; BC = sizeof_e
	push	de		;; ------ ESTOY POR AQUI, INTENTADO SUMAR 1 A HL PARA QUE COPIE BIEN LA MEMORIA
	ld 	d, #0x00
	ld	e, #0x01
	;;ld 	de, #_uno
	add	hl, de	;; hay que anadir 1 a HL
	pop 	de

	;;Como no es la ultima entidad, updateamos la que es ultima entidad
	call update_check_type
	ld 	bc, #sizeof_e
	;; Repeats LDI (LD (DE),(HL), then increments DE, HL, and decrements BC) until BC=0.
	ldir

_last_entity:
	;; falta mover el puntero de la ultima entidad
	ld 	hl, (_last_elem_ptr)
	ld 	bc, #sizeof_e
	sbc	hl, bc	;; hl = la ultima entidad
	ld	(_last_elem_ptr), hl
	;; y restar 1 a la cantidad de entidades
	ld 	a, (_num_entities)	;; incrementamos el numero de enemigos
	dec 	a
	ld    (_num_entities), a

ret



;; ;;;;;;;;;;;;;;;;;;;;;;;;;;; DESTRUYE UN ENEMIGO DEL ARRAY
;; INPUT
;;	HL: Pointer to enemy in the array of the enemies
entityman_destroy_enemy::
	;; Destruirlo de las entidades
	_enemy_to_destroy = #0000
	ld (_enemy_to_destroy), hl
	ld ix, (_enemy_to_destroy)
	;;ld 	(_enemy_to_destroy_IX), hl	;; ponemos en ix el puntero, para sacar la direccion hacia la entidad

	push	hl	;; guardamos el puntero
	;; ponemos en hl el puntero a la entidad en el array de entidades
	ld 	h, ptr_enemy_h(ix)
	ld	l, ptr_enemy_l(ix)	;; ahora tenemos en hl el puntero de esa entidad
	call entityman_destroy_entity

	pop	hl	;; salvamos hl, porque se ha destruido
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;; Tenemos que comprobar si es el ultimo enemigo
	push	hl		;; guardamos hl
	ld 	de, (_last_enemy_ptr)
	ld 	bc, #sizeof_ptr_to_enemy
	add	hl, bc		;;sumamos tamano
	sbc	hl, de	;; si da 0, era el ultimo enemigo
	pop	hl	;; hl = Pointer to enemy 
	jr	z, _last_enemy

_NOT_last_enemy:
	;; tenemos que mover el ultimo enemigo a la posicion de hl
	;; primero calculamos la posicion del ultimo enemigo
	ex	de, hl	;; DE tenia _last_enemy_ptr
				;; ahora DE tiene el enemigo a borrar
	sbc	hl, bc	;; hl = la ultima entidad
				;; BC = sizeof_ptr_to_enemy
	push	de		;; ------ ESTOY POR AQUI, INTENTADO SUMAR 1 A HL PARA QUE COPIE BIEN LA MEMORIA
	ld 	d, #0x00
	ld	e, #0x01
	;;ld 	de, #_uno
	add	hl, de	;; hay que anadir 1 a HL
	pop 	de
	;; Repeats LDI (LD (DE),(HL), then increments DE, HL, and decrements BC) until BC=0.
	ldir

_last_enemy:
	;; falta mover el puntero de la ultima entidad
	ld 	hl, (_last_enemy_ptr)
	ld 	bc, #sizeof_ptr_to_enemy 
	sbc	hl, bc	;; hl = la ultima entidad
	ld	(_last_enemy_ptr), hl
	;; y restar 1 a la cantidad de enemigos
	ld 	a, (_num_enemies)	;; incrementamos el numero de enemigos
	dec 	a
	ld    (_num_enemies), a

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;; mover puntero final y disminuir _num_enemies
	
	;;_enemy_to_destroy_IX = . + 2 		;; mete directamente en ix el valor
	;;ld 	ix, #0x0000
ret



;; ;;;;;;;;;; DESTRUYE UNA BALA DEL ARRAY
;; INPUT
;;	HL: Pointer to bullet in the array of the bullets
entityman_destroy_bullet::
	;; Destruirla de las entidades
	_bullet_to_destroy = #0000
	ld (_bullet_to_destroy), hl
	ld ix, (_bullet_to_destroy)

	push	hl	;; guardamos el puntero
	;; ponemos en hl el puntero a la entidad en el array de entidades
	ld 	h, ptr_enemy_h(ix)
	ld	l, ptr_enemy_l(ix)	;; ahora tenemos en hl el puntero de esa entidad
	call entityman_destroy_entity

	pop	hl	;; salvamos hl, porque se ha destruido
	
	;; Tenemos que comprobar si es la ultima bala
	push	hl		;; guardamos hl
	ld 	de, (_last_bullet_ptr)
	ld 	bc, #sizeof_ptr_to_bullet
	add	hl, bc		;;sumamos tamano
	sbc	hl, de	;; si da 0, era la ultima bala
	pop	hl	;; hl = Pointer to bullet 
	jr	z, _last_bullet

_NOT_last_bullet:
	;; tenemos que mover la ultima bala a la posicion de hl
	;; primero calculamos la posicion de la ultima bala
	ex	de, hl	;; DE tenia _last_bullet_ptr
				;; ahora DE tiene la bala a borrar
	sbc	hl, bc	;; hl = la ultima entidad
				;; BC = sizeof_ptr_to_bullet
	push	de		
	ld 	d, #0x00
	ld	e, #0x01
	add	hl, de	;; hay que anadir 1 a HL
	pop 	de
	;; Repeats LDI (LD (DE),(HL), then increments DE, HL, and decrements BC) until BC=0.
	ldir

_last_bullet:
	;; falta mover el puntero de la ultima entidad
	ld 	hl, (_last_bullet_ptr)
	ld 	bc, #sizeof_ptr_to_bullet 
	sbc	hl, bc	;; hl = la ultima entidad
	ld	(_last_bullet_ptr), hl
	;; y restar 1 a la cantidad de balas
	ld 	a, (_num_bullets)	;; decrementamos el numero de balas
	dec 	a
	ld    (_num_bullets), a

ret




;; ;;;;;;;;;; DESTRUYE UN FUERTE DEL ARRAY
;; INPUT
;;	HL: Pointer to fort in the array of the forts
entityman_destroy_fort::
	;; Destruirla de las entidades
	_fort_to_destroy = #0000
	ld (_fort_to_destroy), hl
	ld ix, (_fort_to_destroy)

	push	hl	;; guardamos el puntero
	;; ponemos en hl el puntero a la entidad en el array de entidades
	ld 	h, ptr_fort_h(ix)
	ld	l, ptr_fort_l(ix)	;; ahora tenemos en hl el puntero de esa entidad
	call entityman_destroy_entity

	pop	hl	;; salvamos hl, porque se ha destruido
	
	;; Tenemos que comprobar si es la ultima bala
	push	hl		;; guardamos hl
	ld 	de, (_last_fort_ptr)
	ld 	bc, #sizeof_ptr_to_fort
	add	hl, bc		;;sumamos tamano
	sbc	hl, de	;; si da 0, era el ultimo fuerte
	pop	hl	;; hl = Pointer to fort 
	jr	z, _last_fort

_NOT_last_fort:
	;; tenemos que mover el ultimo fuerte a la posicion de hl
	;; primero calculamos la posicion del ultimo fuerte
	ex	de, hl	;; DE tenia _last_fort_ptr
				;; ahora DE tiene el fuerte a borrar
	sbc	hl, bc	;; hl = la ultima entidad
				;; BC = sizeof_ptr_to_fort
	push	de		
	ld 	d, #0x00
	ld	e, #0x01
	add	hl, de	;; hay que anadir 1 a HL
	pop 	de
	;; Repeats LDI (LD (DE),(HL), then increments DE, HL, and decrements BC) until BC=0.
	ldir

_last_fort:
	;; falta mover el puntero de la ultima entidad
	ld 	hl, (_last_fort_ptr)
	ld 	bc, #sizeof_ptr_to_fort 
	sbc	hl, bc	;; hl = la ultima entidad
	ld	(_last_fort_ptr), hl
	;; y restar 1 a la cantidad de balas
	ld 	a, (_num_forts)	;; decrementamos el numero de balas
	dec 	a
	ld    (_num_forts), a
ret



;; ---------------------------------------- RESETS



;; REINICIA EL ARRAY DE ENTIDADES
reset_entity_array::
	;;Tenemos que poner a 0 el _num_entities
	xor 	a 			;; a = 0
	ld 	(_num_entities), a

	;; Tengo que mover el _last_elem_ptr al principio del array
	ld 	bc, #_entity_array 
	ld	(_last_elem_ptr), bc
ret


;; REINICIA EL ARRAY DE ENEMIGOS
reset_enemy_array::
	;;Tenemos que poner a 0 el _num_enemies
	xor 	a 			;; a = 0
	ld 	(_num_enemies), a

	;; Tengo que mover el _last_enemy_ptr al principio del array
	ld 	bc, #_enemy_array 
	ld	(_last_enemy_ptr), bc
ret

;; REINICIA EL ARRAY DE BALAS
reset_bullet_array::
	;;Tenemos que poner a 0 el _num_bullets
	xor 	a 			;; a = 0
	ld 	(_num_bullets), a

	;; Tengo que mover el _last_bullet_ptr al principio del array
	ld 	bc, #_bullet_array 
	ld	(_last_bullet_ptr), bc
ret

;; REINICIA EL ARRAY DE FUERTES
reset_fort_array::
	;;Tenemos que poner a 0 el _num_bullets
	xor 	a 			;; a = 0
	ld 	(_num_forts), a

	;; Tengo que mover el _last_fort_ptr al principio del array
	ld 	bc, #_fort_array 
	ld	(_last_fort_ptr), bc
ret




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; ESPECIALES
;; INPUT:
;;    BC: Velocity x, y
;;    HL: Pos x, y
create_bullet::
	;_pos_x .EQU AppBackUpScreen
	;_pos_y .EQU _pos_x+2
	;_vel_x .EQU _pos_y+2
	;_vel_y .EQU _vel_x+2
	
	;_pos_x = #00
	;_pos_y = #00
	;_vel_x = #00
	;_vel_y = #00
	;; Comprobación para que no se cree una bala sin velocidad
	or	b
	jr 	nz, _create_bullet
	or	c
	jr 	nz, _create_bullet
	;; ponemos su velocidad en y a 1
	ld	c, #-4

	_create_bullet:
	ld 	iy, #bull
	ld	a, h
	ld	e_x(iy), a
	ld	a, l
	ld	e_y(iy), a
	ld	a, b
	ld	e_vx(iy), a
	ld	a, c
	ld	e_vy(iy), a
	;DefineEntity  bull,  _pos_x, _pos_y, _vel_x, _vel_y, 4, 16, 0xFF, _bullet_sp, e_ai_st_noAI

	;DefineBullet  bull, _pos_x, _pos_y, _vel_x, _vel_y
	ld 	hl, #bull
    	call entityman_create_bullet
ret

;; INPUT:
;;    BC: Velocity x, y
;;    HL: Pos x, y
create_enemy_bullet::
	;_pos_x .EQU AppBackUpScreen
	;_pos_y .EQU _pos_x+2
	;_vel_x .EQU _pos_y+2
	;_vel_y .EQU _vel_x+2
	
	;_pos_x = #00
	;_pos_y = #00
	;_vel_x = #00
	;_vel_y = #00
	ld 	iy, #enemy_bull
	ld	a, h
	ld	e_x(iy), a
	ld	a, l
	ld	e_y(iy), a
	ld	a, b
	ld	e_vx(iy), a
	ld	a, c
	ld	e_vy(iy), a
	;DefineEntity  bull,  _pos_x, _pos_y, _vel_x, _vel_y, 4, 16, 0xFF, _bullet_sp, e_ai_st_noAI

	;DefineBullet  bull, _pos_x, _pos_y, _vel_x, _vel_y
	ld 	hl, #enemy_bull
    call entityman_create_bullet
ret


;; INPUT:
;;    HL: Pos x, y
;; 	 A: AI Type
create_enemy::

	ld 	ix, #enem
	ld	e_ai_type(ix), a
	ld	a, h
	ld	e_x(ix), a
	ld	a, l
	ld	e_y(ix), a


	ld 	hl, #enem
    	call entityman_create_enemy
ret


;; INPUT:
;;    HL: Pos x, y
;; 	 A: AI Type
create_fort::

	ld 	ix, #fort
	ld	e_ai_type(ix), a
	ld	a, h
	ld	e_x(ix), a
	ld	a, l
	ld	e_y(ix), a


	ld 	hl, #fort
    	call entityman_create_fort
ret

check_player_hp_and_create::
	;;primero comprobar si hay jugador
	call entityman_getEntityArray_IX
	;;comprobamos si ya se ha creado un jugador
	xor	a	;; a = player type
	cp	a, e_type(ix)
	jr	nz, _no_tiene_vida 	;;si no se habia creado

	;; comprobamos si su vida == 0
	ld	a, e_hp(ix)
	cp	a, #0
	jr	nz, _tiene_vida
_no_tiene_vida:
	ld	a, #3
_tiene_vida:
	ld 	iy, #player
	ld	e_hp(iy), a

	ld 	hl, #player
      call entityman_create
ret

 
;;INPUT
;;IX: puntero a posicion en array
;check_hp::
;	call entityman_getEntityArray_IX				
;	ld 	b, a 		;;load in B number of entities
;
;	_update_hp:
;
;		ld a, e_hp(ix) 		;;A = HP(IX)
;		cp #0
;		jr z, _check_type
;		jr _end
;
;	_check_type:
;
;		;push bc
;		;call rendersys_clear_entity
;		;pop bc
;
;		ld__a_ixl
;		ld  l, a 			;; HL = IX
;		ld__a_ixh
;		ld  h, a
;
;		ld a, e_type(ix)
;
;		cp #e_type_enemy
;		jr 	z, _enemy			;;PUSH/POP PARA NO PERDER A
;
;
;		cp #e_type_fort
;		jr 	z, _fort
;
;
;		cp #e_type_bullet
;		jr 	z, _bullet
;
;
;		cp #e_type_player
;		jr 	z, _player
;		ret
;
;	_end:
;		dec 	b 			;; num_entities--
;		ret 	z			;; return if  num_entities = 0
;
;		ld 	de, #sizeof_e 	;; DE = Entity_size
;		add 	ix, de 		;; IX, next entity
;
;		jr _update_hp
;
;	_enemy:
;		call entityman_destroy_enemy
;	ret
;
;	_fort:
;		call entityman_destroy_fort
;	ret
;
;	_bullet:
;		call entityman_destroy_bullet
;	ret
;
;	_player:
;		call gameOver_screen
;	ret
;ret 



check_hp::
	;;primero comprobamos si el jugador esta muerto
	call entityman_getEntityArray_IX
	ld 	b, e_hp(ix) 		;;A = HP del jugador
	xor	a		;; a=0
	cp 	b
	call 	z, gameOver_screen ;; el jugador ha muerto
	
	call check_hp_enemy	;; se mete aqui si no es 0
ret

check_hp_enemy::
	
	call entityman_getEnemyArray_IX
	call entityman_getNumEnemies_A
	cp	#0	;;si no hay enemigos pasa al siguiente
	jr	z, _next_array_enemy

	ld 	b, a 		;;load in B number of enemies

	_update_hp_enemy:
		;;tenemos que acceder a los datos del enemigo
		;;meteremos en iy el puntero a la entidad del enemigo
		ld	a, ptr_enemy_l(ix)
		ld__iyl_a
		ld	a, ptr_enemy_h(ix)
		ld__iyh_a

		;;iy = entitiy del enemigo
		;; ahora comprobamos si su vida es 0
		xor	a	;; a=0
		cp	e_hp(iy)
		jr	z, _destroy_enemy
		jr _end_enemy

	_end_enemy:
		dec 	b 			;; num_enemies--
		jr	z, _next_array_enemy ;; next method if  num_enemies = 0

		ld 	de, #sizeof_ptr_to_enemy 	;; DE = ptr_size
		add 	ix, de 		;; IX, next entity

		jr _update_hp_enemy

	_destroy_enemy:
		;; cargamos en hl el puntero al array de balas
		ld__a_ixl
		ld  l, a 			;; HL = IX
		ld__a_ixh
		ld  h, a
		call entityman_destroy_enemy
		call down_you_go
_next_array_enemy:
		call check_hp_bullet
	
ret


check_hp_bullet::
	
	call entityman_getBulletArray_IX
	call entityman_getNumBullets_A
	cp	#0	;;si no hay balas pasa al siguiente
	jr	z, _next_array_bullet

	ld 	b, a 		;;load in B number of enemies

	_update_hp_bullet:
		;;tenemos que acceder a los datos del enemigo
		;;meteremos en iy el puntero a la entidad del enemigo
		ld	a, ptr_bullet_l(ix)
		ld__iyl_a
		ld	a, ptr_bullet_h(ix)
		ld__iyh_a

		;;iy = entitiy del enemigo
		;; ahora comprobamos si su vida es 0
		xor	a	;; a=0
		cp	e_hp(iy)
		jr	z, _destroy_bullet
		jr _end_bullet

	_end_bullet:
		dec 	b 			;; num_enemies--
		jr	z, _next_array_bullet ;; next method if  num_enemies = 0

		ld 	de, #sizeof_ptr_to_bullet 	;; DE = ptr_size
		add 	ix, de 		;; IX, next entity

		jr _update_hp_bullet

	_destroy_bullet:
		;; cargamos en hl el puntero al array de balas
		ld__a_ixl
		ld  l, a 			;; HL = IX
		ld__a_ixh
		ld  h, a
		call entityman_destroy_bullet
_next_array_bullet:
		call check_hp_fort
ret


check_hp_fort::
	
	call entityman_getFortArray_IX
	call entityman_getNumForts_A
	cp	#0	;;si no hay fuertes acaba
	ret	z

	ld 	b, a 		;;load in B number of enemies

	_update_hp_fort:
		;;tenemos que acceder a los datos del enemigo
		;;meteremos en iy el puntero a la entidad del enemigo
		ld	a, ptr_fort_l(ix)
		ld__iyl_a
		ld	a, ptr_fort_h(ix)
		ld__iyh_a

		;;iy = entitiy del enemigo
		;; ahora comprobamos si su vida es 0
		xor	a	;; a=0
		cp	e_hp(iy)
		jr	z, _destroy_fort
		jr _end_fort

	_end_fort:
		dec 	b 			;; num_enemies--
		ret 	z		 ;; fin if  num_forts = 0

		ld 	de, #sizeof_ptr_to_fort 	;; DE = ptr_size
		add 	ix, de 		;; IX, next entity

		jr _update_hp_fort

	_destroy_fort:
		;; cargamos en hl el puntero al array de balas
		ld__a_ixl
		ld  l, a 			;; HL = IX
		ld__a_ixh
		ld  h, a

		call entityman_destroy_fort
ret


;;Comprueba de que tipo es _last_entity, para actualizarlo a su nueva posicion
;; DESTROYS: IX, A, BC
;; INPUT: 	HL, _last_entity
;;		DE, new position (posicion de la entidad que se borra)
update_check_type::
	;; meter en ix, de
	ld	a, l
	ld__ixl_a
	ld	a, h
	ld__ixh_a
	;;accedemos con ix a los datos de la entidad para ver el tipo
	ld	a, e_type(ix)
	cp	#e_type_enemy
	jr	nc, _no_bullet	;; if a >= ix(tipo), no es bala (balas son 1 y 2)
_bullet:
	call update_bullet_array
	ret
_no_bullet:

	ld	a, #e_type_enemy
	cp	e_type(ix)
	jr	nz, _fort	;; if ix(tipo) != 3, es un fuerte
_enemy:
	call update_enemy_array
	ret
_fort:
	call update_fort_array
ret


;;Actualiza el puntero de la bala que equivale a la ultima entidad y que se va a cambiar de sitio
update_bullet_array::
	call entityman_getBulletArray_IX
	call entityman_getNumBullets_A

	ld 	b, a 		;;load in B number of bullets

	_update_bullet:
		;; comprobamos si DE == esta_bala
		ld	a, ptr_bullet_h(ix)	;; a=(h de ix)
		cp	h
		jr	nz, _no_igual_bullet	;; si es igual, vemos la parte h

		ld	a, ptr_bullet_l(ix)	;; a=(h de ix)
		cp	l
		jr	nz, _no_igual_bullet	;; si es igual, vemos la parte l
	_igual_bullet:
		;;si es la misma, cambiamos sus valores por los de hl
		ld	ptr_bullet_h(ix), d
		ld	ptr_bullet_l(ix), e
		ret

	_no_igual_bullet:
		dec 	b
		ret	z	 ;;acaba si no ha encontrado nada (nunca deberia de entrar aqui)
		push 	de	;; se va a destruir
		ld 	de, #sizeof_ptr_to_enemy 	;; DE = ptr_size
		add 	ix, de 		;; IX, next entity
		pop	de
		jr _update_bullet
ret


;;Actualiza el puntero del enemigo que equivale a la ultima entidad y que se va a cambiar de sitio
update_enemy_array::
	call entityman_getEnemyArray_IX
	call entityman_getNumEnemies_A

	ld 	b, a 		;;load in B number of enemies

	_update_enemy:
		;; comprobamos si DE == este_enemigo
		ld	a, ptr_enemy_h(ix)	;; a=(h de ix)
		cp	h
		jr	nz, _no_igual_enemy	;; si es igual, vemos la parte h

		ld	a, ptr_enemy_l(ix)	;; a=(h de ix)
		cp	l
		jr	nz, _no_igual_enemy	;; si es igual, vemos la parte l
	_igual_enemy:
		;;si es la misma, cambiamos sus valores por los de hl
		ld	ptr_enemy_h(ix), d
		ld	ptr_enemy_l(ix), e
		ret

	_no_igual_enemy:
		dec 	b
		ret	z	 ;;acaba si no ha encontrado nada (nunca deberia de entrar aqui)
		push 	de	;; se va a destruir
		ld 	de, #sizeof_ptr_to_enemy 	;; DE = ptr_size
		add 	ix, de 		;; IX, next entity
		pop	de
		jr _update_enemy
ret


;;Actualiza el puntero del fuerte que equivale a la ultima entidad y que se va a cambiar de sitio
update_fort_array::
	call entityman_getFortArray_IX
	call entityman_getNumForts_A

	ld 	b, a 		;;load in B number of forts

	_update_fort:
		;; comprobamos si DE == este_enemigo
		ld	a, ptr_fort_h(ix)	;; a=(h de ix)
		cp	h
		jr	nz, _no_igual_fort	;; si es igual, vemos la parte h

		ld	a, ptr_fort_l(ix)	;; a=(h de ix)
		cp	l
		jr	nz, _no_igual_fort	;; si es igual, vemos la parte l
	_igual_fort:
		;;si es la misma, cambiamos sus valores por los de hl
		ld	ptr_fort_h(ix), e
		ld	ptr_fort_l(ix), e
		ret

	_no_igual_fort:
		dec 	b
		ret	z	 ;;acaba si no ha encontrado nada (nunca deberia de entrar aqui)
		push 	de	;; se va a destruir
		ld 	de, #sizeof_ptr_to_fort 	;; DE = ptr_size
		add 	ix, de 		;; IX, next entity
		pop	de
		jr _update_fort
ret


;; INPUT		
;;  IX, fort that needs to be rendered		
fort_needs_render::		
	ld	a, #1		
	ld	e_render_fort(ix), a		
ret


all_forts_needs_render::		
   call entityman_getFortArray_IY
   call entityman_getNumForts_A
   cp #0 ;;si no hay fuertes acaba
   ret   z

   ld    b, a     ;;load in B number of forts

   _check_next_fort:
      ;;tenemos que acceder a los datos del fuerte
      ;;meteremos en iy el puntero a la entidad del fuerte
      ld a, ptr_fort_l(iy)
      ld__ixl_a
      ld a, ptr_fort_h(iy)
      ld__ixh_a

      ;;poner a que se tiene que renderizar
      ld	a, #1
      ld    e_render_fort(ix), a

   _next_fort:
      dec   b        ;; num_forts--
      ret   z      ;; fin if  num_forts = 0

      ld    de, #sizeof_ptr_to_fort    ;; DE = ptr_size
      add   iy, de      ;; IX, next entity

      jr _check_next_fort
ret


;--------------------------------- INSERVIBLE
;; Comprueba todas las entidades de los arrays y actualiza los punteros 
;;   de las entidades en posiciones superiores a la entidad borrada (resta sizeof_ptr)
;; INPUT: HL pointer to destroyed entity
;update_entities::
;	call update_entities_enemies
;ret
;update_entities_enemies::
;	;; Se comprueba el array de enemigos
;	call entityman_getEnemyArray_IX
;	call entityman_getNumEnemies_A
;	cp	#0	;;si no hay enemigos pasa al siguiente
;	jr	z, _update_next_array_enemy
;
;	ld 	b, a 		;;load in B number of enemies
;
;	_update_enemy:
;		;;tenemos que acceder a los datos del enemigo
;		ld	a, ptr_enemy_h(ix)	;; a=(h de ix)
;		cp	h
;		jr	c, _es_menor_enemy	;; no hago nada
;
;		ld	a, ptr_enemy_l(ix)	;; a=(h de ix)
;		cp	l
;		jr	z, _es_menor_enemy	;; esto es cuando es igual (es la que se esta destruyendo), no hago nada
;		jr	c, _es_menor_enemy	;; si es menos, no hago nada
;
;	_es_mayor_enemy:
;	push	hl	;; no tengo que cargarme hl
;		;; meto en hl el valor del puntero que quiero restar
;		ld	h, ptr_enemy_h(ix)
;		ld	l, ptr_enemy_l(ix)
;		ld	de, #sizeof_e
;		sbc hl,de
;		;; acutalizo
;		ld	ptr_enemy_h(ix), h
;		ld	ptr_enemy_l(ix), l
;	pop 	hl
;
;
;		_es_menor_enemy:
;			;; pasamos a la siguiente entidad
;	_next_enemy:
;		dec 	b 			;; num_enemies--
;		jr	z, _update_next_array_enemy ;; next method if  num_enemies = 0
;
;		ld 	de, #sizeof_ptr_to_enemy 	;; DE = ptr_size
;		add 	ix, de 		;; IX, next entity
;
;		jr _update_enemy
;
;_update_next_array_enemy:
;		call update_entities_bullets
;ret
;
;update_entities_bullets::
;	;; Se comprueba el array de balas
;	call entityman_getBulletArray_IX
;	call entityman_getNumBullets_A
;	cp	#0	;;si no hay balas pasa al siguiente
;	jr	z, _update_next_array_bullet
;
;	ld 	b, a 		;;load in B number of bullets
;
;	_update_bullet:
;		ld	a, ptr_bullet_h(ix)	;; a=(h de ix)
;		cp	h
;		jr	c, _es_menor_bullet	;; no hago nada
;
;		ld	a, ptr_bullet_l(ix)	;; a=(h de ix)
;		cp	l
;		jr	z, _es_menor_bullet	;; esto es cuando es igual (es la que se esta destruyendo), no hago nada
;		jr	c, _es_menor_bullet	;; si es menos, no hago nada
;
;	_es_mayor_bullet:
;	push	hl	;; no tengo que cargarme hl
;		;; meto en hl el valor del puntero que quiero restar
;		ld	h, ptr_bullet_h(ix)
;		ld	l, ptr_bullet_l(ix)
;		ld	de, #sizeof_e
;		sbc hl,de
;		;; acutalizo
;		ld	ptr_bullet_h(ix), h
;		ld	ptr_bullet_l(ix), l
;	pop 	hl
;
;
;		_es_menor_bullet:
;			;; pasamos a la siguiente entidad
;	_next_bullet:
;		dec 	b 			;; num_enemies--
;		jr	z, _update_next_array_bullet ;; next method if  num_enemies = 0
;
;		ld 	de, #sizeof_ptr_to_bullet 	;; DE = ptr_size
;		add 	ix, de 		;; IX, next entity
;
;		jr _update_bullet
;
;_update_next_array_bullet:
;		call update_entities_forts
;ret
;
;update_entities_forts::
;	;; Se comprueba el array de balas
;	call entityman_getFortArray_IX
;	call entityman_getNumForts_A
;	cp	#0	;;si no hay fuertes acaba
;	ret	z
;
;	ld 	b, a 		;;load in B number of forts
;
;	_update_fort:
;		ld	a, ptr_fort_h(ix)	;; a=(h de ix)
;		cp	h
;		jr	c, _es_menor_fort	;; no hago nada
;
;		ld	a, ptr_fort_l(ix)	;; a=(h de ix)
;		cp	l
;		jr	z, _es_menor_fort	;; esto es cuando es igual (es la que se esta destruyendo), no hago nada
;		jr	c, _es_menor_fort	;; si es menos, no hago nada
;
;	_es_mayor_fort:
;	push	hl	;; no tengo que cargarme hl
;		;; meto en hl el valor del puntero que quiero restar
;		ld	h, ptr_fort_h(ix)
;		ld	l, ptr_fort_l(ix)
;		ld	de, #sizeof_e
;		sbc hl,de
;		;; acutalizo
;		ld	ptr_fort_h(ix), h
;		ld	ptr_fort_l(ix), l
;	pop 	hl
;
;
;		_es_menor_fort:
;			;; pasamos a la siguiente entidad
;	_next_fort:
;		dec 	b 			;; num_forts--
;		ret	z 			;; acaba si no hay mas fuertes
;
;		ld 	de, #sizeof_ptr_to_fort 	;; DE = ptr_size
;		add 	ix, de 		;; IX, next entity
;
;		jr _update_fort
;ret