;  ----------------------------LICENSE NOTICE-----------------------------
;
;  This file is part of the Hey Sailor! game for Amstrad CPC 464.
;  Copyright (C) 2019  Daniel Bravo García  (dani_jijona@hotmail.com)
;  Copyright (C) 2019  David Navas Díaz   (dnavasdiaz97@gmail.com)
;  Copyright (C) 2019  Juan Manuel Ruiz Culiañez  (pokejuanma@gmail.com)
;
;  Hey Sailor! is free software: you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  Hey Sailor! is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with Hey Sailor!. If not, see <http://www.gnu.org/licenses/>.
;
; ----------------------------------------------------------------------



;;
;; Game Manager
;;

.include "sys/render_system.h.s"
.include "man/entity_manager.h.s"
.include "cpct_functions.h.s"
.include "sys/physics_system.h.s"
.include "sys/input_system.h.s"
.include "sys/ai_control.h.s"
.include "man/patrol.h.s"
.include "sys/collision.h.s"
.include "sys/animation_system.h.s"
.include "man/level_manager.h.s"
.include "assets/music/music.h.s"
.include "man/game_manager.h.s"

; Includes para los mapas comprimidos
.include "src/assets/compressed/mimapa.h.s"
.include "src/assets/compressed/mifotoinicio.h.s"

_level: .db 0
_cooldown_hit: .db 0

game_man_init::

  call rendersys_init

  ;;Ponemos el fondo del juego (mapa de juego)
  ld hl, #_mimapa_end
  call rendersys_set_background

  ;; Init systems
  call man_patrol_init
  call entityman_init

  ld  a, (_level)
  call level_loader

  call change_lives_sprites
  ld  a, (_level)
  call render_num_level
  call render_face_sailor

  call entityman_getEntityArray_IX
  call entityman_getNumEntities_A
  call sys_ai_control_init

  ;call physicsys_init ;; Lo llamamos pero todavia no hace nada (ni va a hacer loco)
ret

game_man_start::
    ;xor   a
    ;ld  (_level), a
    call rendersys_init

    
    ;; Ponemos la pantalla de inicio
    ld hl,  #_mifotoinicio_end ;;ESTO VA AQUI Y NO EN INPUT_SCREEN PORQUE SI NO SE RALEENTIZA Y LA MUSICA VA COMO LA MIERDA Y TARDA UN HUEVO EN PILLAR EL ENTER
    call rendersys_set_background

     ;;Ponemos la musica de la pantalla de inicio
      ld de, #_intro_song
      call cpct_akp_musicInit_asm
      call input_screen ;; LLAMADA A LA PANTALLA DE INICIO PARA PRESS START POR EJEMPLO

    ;;Ponemos el fondo del juego (mapa de juego)
    ld hl, #_mimapa_end
    call rendersys_set_background


ret

game_man_update::
    call game_man_next_level ;; comprobamos si nos hemos pasado el nivel

    call entityman_getEntityArray_IX
    call sys_input_update
    call render_update_sailor_face

    ;;cooldown para cuando te quitan vida y te teletransportas a la posicion inicial   
    ld  a, (_cooldown_hit)    
    cp  #0    
    jr  nz, _no_updates
    
    call entityman_getEntityArray_IX
    call entityman_getNumEntities_A
    call sys_ai_control_update


    call physicsys_destroy_bullets_update ;; destruimos las balas que tocan borde
    call physicsys_teleport
    call entityman_getEntityArray_IX
  	call physicsys_update ;; Implantamos el movimiento desde physics_system
    
    _no_updates:
    call animation_update_player

    call sys_collision_update


    call check_hp
    call check_hp   ;; llamamos 2 veces a esta funcion porque se pueden haber destruido dos objetos


    ;; para decrementar el contador    
    ld  a, (_cooldown_hit)    
    cp  #0    
    ret z   
    dec a   
    ld  (_cooldown_hit), a
ret

game_man_render::
;; TODO: Poner una variable en la que guardemos el array en el init (llamar desde init)
      call rendersys_calculate_pos
      call entityman_getEntityArray_IX  ;; cargamos en IX el array a entiades
      call entityman_getNumEntities_A   ;; cargamos en A  el numero de entidades
      call rendersys_update

      call check_render_forts
ret

game_man_restart::
  xor   a
  ld  (_level), a
  call game_man_init
ret

;;Comprueba si nos hemos pasado el nivel y pasa al siguiente si es asi
game_man_next_level::
  ;; comprueba si hay 0 fuertes y enemigos para pasar de nivel
  call entityman_getNumForts_A
  cp  #0
  jr  nz, _nada
_check_enemies:
  call entityman_getNumEnemies_A
  cp  #0
  jr  nz, _nada

_next_level: ;; Aqui metemos el pantallazo de stage_clear

  call cpct_akp_stop_asm ;;Paramos la musica

  ld  a, (_level)
  inc a
  cp  a, #n_levels ;; si ya nos hemos pasado el ultimo nivel
  jr  c, _not_end_game  ;; if a < n_levels

_end_game:
  ;;reiniciamos las vidas del jugador
  call entityman_getEntityArray_IX
  ld  a, #3
  ld  e_hp(ix), a
;; Aqui metemos pantallazo you win!
  call you_win_screen
  xor a
  ld  (_level), a
  call game_man_start
  call game_man_init
  jr _nada
  
_not_end_game:
  ld  (_level), a
  call stage_clear_screen
  call game_man_init
_nada:
ret

      
game_man_activate_cooldown::    
   ld  a, #30    
   ld  (_cooldown_hit), a    
ret   
      
game_man_desactivate_cooldown::   
   ;; reseteamos el contador   
   xor a   
   ld  (_cooldown_hit), a    
 ret




