;  ----------------------------LICENSE NOTICE-----------------------------
;
;  This file is part of the Hey Sailor! game for Amstrad CPC 464.
;  Copyright (C) 2019  Daniel Bravo García  (dani_jijona@hotmail.com)
;  Copyright (C) 2019  David Navas Díaz   (dnavasdiaz97@gmail.com)
;  Copyright (C) 2019  Juan Manuel Ruiz Culiañez  (pokejuanma@gmail.com)
;
;  Hey Sailor! is free software: you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  Hey Sailor! is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with Hey Sailor!. If not, see <http://www.gnu.org/licenses/>.
;
; ----------------------------------------------------------------------



;; Entity AI Control system

.include "man/entity_manager.h.s"
.include "man/patrol.h.s"
.include "cpctelera.h.s"
.include "src/assets/assets.h.s"
.include "sys/animation_system.h.s"

_radius_fort_spawn_x == 320	;; radio en el que los fuertes te pueden mandar un enemigo en x
_radius_fort_spawn_y == 200	;; radio en el que los fuertes te pueden mandar un enemigo en y
;; es el doble de grande porque los pixeles en y son mas pequenos


shot_angle: .db 1  			;; Modo de disparo ( 0 = +   ó   0 = x )
fort_shot_cd: .db 50
enemy_shot_cd: .db 50


sys_ai_control_init::
;	ld	(_ent_counter), a
	ld(_ent_array_ptr_temp_standby), ix
	ld(_ent_array_ptr), ix	;;[6]
	ret

;; INPUT: IX array entity
;;			A, number entities
sys_ai_control_update::
	ld	(_ent_counter), a	

	_ent_array_ptr	= . + 2
		ld ix, #0x0000		;[4]

	_loop:

	ld a, e_ai_type(ix)      ;; Compruebo el tipo de IA
	cp #e_ai_type_no_ai 
	call nz, sys_ai_type_check

	_ent_counter = . + 1
		ld  a, #0
		dec a
		ret z

	ld	(_ent_counter), a

	ld 	de, #sizeof_e
	add	ix, de 		;;Apuntar a la segunda entidad

	jr		_loop


;;INPUT: IX -> Actual entity
;;     :  A -> IA Type
sys_ai_type_check::

	cp #e_ai_type_shot_enemy ;;Compruebo el tipo de entidad, si es follow enemigo hago que siga al player
	jr nz, _no_shot_ai_enemy
	call sys_ai_enemy_follow
	call sys_ai_enemy_shot
	jr _end_ai

	_no_shot_ai_enemy:
	cp #e_ai_type_follow_enemy ;;Compruebo el tipo de entidad, si es follow enemigo hago que siga al player
	jr nz, _no_follow_enemy
	call sys_ai_enemy_follow
	jr _end_ai

	_no_follow_enemy:
	cp #e_ai_type_combo_fort ;; Miro si es fuerte combo
	jr nz, _no_combo_fort    ;; Si no lo es compruebo el siguiente tipo
	call sys_ai_fort_spawn   ;;
	call sys_ai_fort_shot    ;; Si lo es llamo al disparo y al spawn
	jr _end_ai               ;; Termina la IA

	_no_combo_fort:
	cp #e_ai_type_spawn_fort ;; Compruebo si es un fuerte de spawneo
	jr nz, _no_spawn_fort    ;; Si no lo es voy al ultimo tipo
	call sys_ai_fort_spawn   ;; Si lo es Spawnea
	jr _end_ai               ;; Si lo era termina IA

	_no_spawn_fort:
	cp #e_ai_type_bullet_fort
	call z, sys_ai_fort_shot ;; Si es fuerte de disparo

	_end_ai:

ret

sys_ai_enemy_shot::
	call entityman_getEntityArray_IY ;; IY = Player

    ld  a, (enemy_shot_cd)         ;; Comprueba el CD
    cp  #0                         ;; Si el CD = 0 --> Disparo
    jp   nz, _no_shot_enemy       

    ld a, #50                      ;; Vuelvo a actualizar el cd
    ld (enemy_shot_cd), a; 


    ld	a, e_x(ix)                 ;; A = Pos x enemy
    sub #8                         ;; A = X enemy centrada
    cp e_x(iy)                     ;; Z -> A == e_x || C -> A <  e_x || NC -> A >= e_x

    jr nc, x_greater_equal
    jr x_1

    x_greater_equal:
    	jr z, x_0
    	sub #8
    	cp e_x(iy)
    	jr c, x_0

    xsub8_greater_equal:
    	cp e_x(iy)
    	jr z, x_0
    	add #16
    	cp e_x(iy)
    	jr nc, xadd8_greater_equal

    xadd8_greater_equal:
    	cp e_x(iy)
    	jr z, x_0
    	jr x_n1

    x_0:
    	ld b, #0
    	jr _check_y
    x_1:
    	ld b, #2
    	jr _check_y
    x_n1:
    	ld b, #-2

    _check_y:
    	ld c, #0

    ld	a, e_y(ix)                 ;; A = Pos y enemy
    sub #8                         ;; A = X enemy centrada
    cp e_y(iy)                     ;; Z -> A == e_x || C -> A <  e_x || NC -> A >= e_x

    jr nc, y_greater_equal
    jr y_1

    y_greater_equal:
    	jr z, y_0
    	sub #8
    	cp e_y(iy)
    	jr  nc, ysub8_greater_equal

    ysub8_greater_equal:
    	cp e_y(iy)
    	jr z, y_0
    	add #16
    	cp e_y(iy)
    	jr nc, yadd8_greater_equal

    yadd8_greater_equal:
    	cp e_y(iy)
    	jr z, y_0
    	jr y_n1

    y_0:
    	ld c, #0
    	jr _end_check
    y_1:
    	ld c, #2
    	jr _end_check
    y_n1:
    	ld c, #-2

	
	_end_check:

	ld	a, e_x(ix)
	ld  h, a
  	ld	a, e_y(ix)
	ld  l, a

    call create_enemy_bullet ;; HL = Posicion de salida ;; BC = vx y vy bala

	jr   _end_shot_enemy

	_no_shot_enemy:
		dec a                    ;; Si el CD > 0 lo decremento
		ld (enemy_shot_cd), a


	_end_shot_enemy:
ret

sys_ai_enemy_follow::
	_ent_array_ptr_temp_standby = . + 2
	ld iy, #0x0000		;[4]

	ld	a, e_x(iy)
	ld  e_ai_aim_x(ix), a
	ld	a, e_y(iy)
	ld  e_ai_aim_y(ix), a 				;;IA TIENE OBJETIVO (NECESITA COOLDOWN)

	ld  a, e_ai_aim_x(ix)				;;OBJETIVO EN X
	sub e_x(ix)							;;A = DISTANCIA EN X
	ld  b, a 							;;B = DISTANCIA EN X
	
	ld  a, e_ai_aim_y(ix)				;;OBJETIVO EN Y
	sub e_y(ix)							;;A = DISTANCIA EN Y

	sub b 								;;A = Y - X
	jr nc, _y_greater					;; Y > X
	jr _x_greater

	_y_greater:
		ld	e_vx(ix), #-0
		;;UPDATE Y
		ld	a, e_ai_aim_y(ix)			;;A = DIS OBJ Y
		sub	e_y(ix)						;;A = DIS OBJ Y - POS Y
		jr	nc, _objy_greater_or_equal  ;;NO ACARREO => OBJ Y esta más abajo o igual;;

		;;SI ESTA MAS ARRIBA
		_objy_lesser:
			call animation_enemy_Up

			ld	e_vy(ix), #-1
			jr 	_endif_y;;

		;;SI ESTA IGUAL O MAS ABAJO
		_objy_greater_or_equal:
			jr	z, _arrived_y	;Equal

			call animation_enemy_Down

			ld	e_vy(ix), #1
			jr  _endif_y;
		;;SI HA LLEGADO
		_arrived_y:
			ld	e_vy(ix), #0
		_endif_y:
		jr _end_follow
	_x_greater:
		ld	e_vy(ix), #-0
		;;;UPDATE X
		ld	a, e_ai_aim_x(ix)				;A = DIS OBJ X
		sub	e_x(ix)							;;A = DIS OBJ X - POS X
		jr	nc, _objx_greater_or_equal		;;NO ACARREO => OBJ X esta más DCHA o igual;;

		;;SI ESTA MAS A LA IZQUIERDA
		_objx_lesser:
			call animation_enemy_Left

  			ld e_sp_h(ix), h
  			ld e_sp_l(ix), l					
			
			ld	e_vx(ix), #-1				
			jr 	_endif_x

		;;SI ESTA EN LA MISMA POS O MAS ALA DERECHA
		_objx_greater_or_equal:
			jr	z, _arrived_x	;Equal

			call animation_enemy_Right

			ld	e_vx(ix), #1
			jr  _endif_x

		;;SI HA LLEGADO
		_arrived_x:
			ld	e_vx(ix), #0
		_endif_x:
			jr _end_follow
	_end_follow:
	ret







;; IA DE LOS FUERTES, SPAWNEAN ENEMIGOS
;; 	INPUT: IX, fuerte
sys_ai_fort_spawn::
	;; vemos si el contador del fuerte esta a 0, si no, se decrece
	xor   a     ;;a=0
  	cp    a, e_spawn_time(ix) ;;vemos si el contador esta a 0
	jr    z, es_0

no_es_0:
	;; decrementamos y acabamos
	ld    a, e_spawn_time(ix)
  	dec   a
  	ld    e_spawn_time(ix), a
  	ret
es_0:
	;;tenemos que ver si el jugador está a menos de cierta distancia y spawnear un enemigo si el contador esta a 0
	call entityman_getEntityArray_IY	;;cargamos al jugador en IY

	;; PARA X
	;;calculamos la diferencia de posiciones
	ld	a, e_x(iy)	;; a = x del player
	sub	a, e_x(ix)	;; a = diferencia, (da problema si x del player es mayor?)
	call  absA         ;; esto hara que si es negativo, lo haga positivo

	;; ahora comprobamos la diferencia con _radius_fort_spawn
	cp	#_radius_fort_spawn_x
	jr	nc, _end		;; if a < radius

	;; PARA Y
	;;calculamos la diferencia de posiciones
	ld	a, e_y(iy)	;; a = y del player
	sub	a, e_y(ix)	;; a = diferencia, (da problema si y del player es mayor?)
	call  absA         ;; esto hara que si es negativo, lo haga positivo

	;; ahora comprobamos la diferencia con _radius_fort_spawn
	cp	#_radius_fort_spawn_y
	jr	nc, _end		;; if a < radius

_spawn:
	;;reseteamos el contador
  	ld	a, #spawn_timer
  	ld    e_spawn_time(ix), a ;; esto no va y no se porque

	;; si llegamos aqui es porque el player esta dentro del radio
  	;;SPAWNEO
  	ld	h, e_x(ix)
  	ld	l, e_y(ix)		;; meto la posicion del fuerte en hl
  	ld 	a, #e_ai_type_follow_enemy
  	call 	create_enemy	;; creo el enemigo

_end:	;; no se ha metido ni por x ni por y

ret

;; FUNCION PARA HACER ABS DE A
;; INPUT: A cualquier valor
absA:
     or a
ret p
     neg         ;or you can use      cpl \ inc a
ret

.macro HLandBullet
	ld	a, e_x(ix)
	add a, #4			;; X + 4 para que sea en el centro del fuerte
	ld  h, a

  	ld	a, e_y(ix)		;; Y + 16 para que sea en el centro del fuerte
	add a, #16
	ld  l, a

	call create_enemy_bullet

.endm

sys_ai_fort_shot::
    ld  a, (fort_shot_cd)         ;; Si es un fuerte comprueba el CD
    cp  #0                   ;; Si el CD = 0 --> Disparo
    jp   nz, _no_shot

    ld a, #50                ;;Vuelvo a actualizar el cd
    ld (fort_shot_cd), a;

	ld a, (shot_angle)        ;; A = Tipo de Disparo
	cp #0
	jr z, _edge               ;; Salto al respectivo salto 

	_ex:
	;;Arriba Izquierda
	ld b, #-1             
	ld c, #-1
	HLandBullet

	;;Abajo Izquierda
	ld b, #-1   
	ld c, #1
	HLandBullet

	;;Abajo Derecha
	ld b, #1   
	ld c, #-1
	HLandBullet

	;;Arriba Derecha
	ld b, #1   
	ld c, #1
	HLandBullet

	ld a, (shot_angle)    ;; A = Tipo de disparo ( 1  al ser Equis)
	dec a                 ;; A = 0 --> Cruz
	ld (shot_angle), a 
	jr   _end_shot

	_edge:
	;;Arriba
	ld b, #0   
	ld c, #-2
	HLandBullet

	;;Izquierda
	ld b, #-1   
	ld c, #0
	HLandBullet

	;;Abajo
	ld b, #0   
	ld c, #2
	HLandBullet
	
	;;Derecha
	ld b, #1   
	ld c, #0
	HLandBullet

	ld a, (shot_angle)
	inc a
	ld (shot_angle), a
	jr _end_shot

	_no_shot:
		dec a                    ;; Si el CD > 0 lo decremento
		ld (fort_shot_cd), a

	_end_shot:

ret