;  ----------------------------LICENSE NOTICE-----------------------------
;
;  This file is part of the Hey Sailor! game for Amstrad CPC 464.
;  Copyright (C) 2019  Daniel Bravo García  (dani_jijona@hotmail.com)
;  Copyright (C) 2019  David Navas Díaz   (dnavasdiaz97@gmail.com)
;  Copyright (C) 2019  Juan Manuel Ruiz Culiañez  (pokejuanma@gmail.com)
;
;  Hey Sailor! is free software: you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  Hey Sailor! is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with Hey Sailor!. If not, see <http://www.gnu.org/licenses/>.
;
; ----------------------------------------------------------------------



;;ANIMATION SYSTEM

.include "assets/assets.h.s"
.include "man/entity_manager.h.s"
.include "cpctelera.h.s"
.include "sys/render_system.h.s"


;;Funciones auxiliares
.macro counter_plus ;;Suma 1 al contador jaja
    ld a, e_animation(ix)
    inc a
    ld e_animation(ix), a
.endm

.macro counter_reset ;; Resetea a 0 el contador jajaja
  ld a, #0
  ld e_animation(ix), a
.endm

.macro dec_cd
    ld a, e_anim_cd(ix)
    dec a
    ld e_anim_cd(ix), a
.endm

.macro reset_cd
  ld e_anim_cd(ix), #2
.endm

animation_update_player::
  call entityman_getEntityArray_IX         ;; IX = player

  ld a, e_anim_cd(ix)
  cp #0
  jr nz, _end
  reset_cd


  ld a, e_vx(ix)
  cp #0                                    ;; Si la velocidad horizontal es 0, entonces se mueve en vertical
  jr z, _vertical


  _horizontal:
  ld a, e_vx(ix)
  cp #1                                    ;; Si la velocidad horizontal es 1 se mueve hacia la derecha
  jr z, _right
    _left:
    call animation_player_Left             ;; Si no se mueve hacia la izquierda
    jr _end
    _right:
    call animation_player_Right
    jr _end


  _vertical:
  ld a, e_vy(ix)
  cp #2
  jr z, _down
    _up:
    call animation_player_Up
    jr _end
    _down:
    call animation_player_Down

  _end:
  dec_cd
ret

;;FUNCIONES DE ANIMACION PARA EL PERSONAJE
animation_player_Up::
ld a, e_animation(ix)
cp #0 ;; Comparamos con registro a
jr nz, animation_Up_02 ;; Z si es igual y NZ si no es igual

  animation_Up_01:
    ld hl, #_player2_sp_01 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==1
      jr fin_animation_player_Up 

    animation_Up_02:
      ld a, e_animation(ix)
      cp #1 ;; Comparamos con registro a
      jr nz, animation_Up_03 ;; Z si es igual y nz si no es igual

      ld hl, #_player2_sp_02 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==2
      jr fin_animation_player_Up ;;terminamos

    animation_Up_03:
      ld hl, #_player2_sp_03 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_reset

    fin_animation_player_Up:
      
ret

animation_player_Left::
ld a, e_animation(ix)
cp #0 ;; Comparamos con registro a
jr nz, animation_Left_02 ;; Z si es igual y NZ si no es igual

  animation_Left_01:
    ld hl, #_player2_sp_05 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==1
      jr fin_animation_player_Left 

    animation_Left_02:
      ld a, e_animation(ix)
      cp #1 ;; Comparamos con registro a
      jr nz, animation_Left_03 ;; Z si es igual y nz si no es igual

      ld hl, #_player2_sp_06 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==2
      jr fin_animation_player_Left ;;terminamos

    animation_Left_03:
      ld hl, #_player2_sp_07 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_reset

    fin_animation_player_Left:

ret

animation_player_Right::
ld a, e_animation(ix)
cp #0 ;; Comparamos con registro a
jr nz, animation_Right_02 ;; Z si es igual y NZ si no es igual

  animation_Right_01:
    ld hl, #_player2_sp_09 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==1
      jr fin_animation_player_Right 

    animation_Right_02:
        ld a, e_animation(ix)
      cp #1 ;; Comparamos con registro a
      jr nz, animation_Right_03 ;; Z si es igual y nz si no es igual

      ld hl, #_player2_sp_10 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==2
      jr fin_animation_player_Right ;;terminamos

    animation_Right_03:
      ld hl, #_player2_sp_11 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_reset

    fin_animation_player_Right:

ret

animation_player_Down::
ld a, e_animation(ix)
cp #0 ;; Comparamos con registro a
jr nz, animation_Down_02 ;; Z si es igual y NZ si no es igual

  animation_Down_01:
    ld hl, #_player2_sp_13 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==1
      jr fin_animation_player_Down 

    animation_Down_02:
        ld a, e_animation(ix)
      cp #1 ;; Comparamos con registro a
      jr nz, animation_Down_03 ;; Z si es igual y nz si no es igual

      ld hl, #_player2_sp_14 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==2
      jr fin_animation_player_Down ;;terminamos

    animation_Down_03:
      ld hl, #_player2_sp_15 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_reset

    fin_animation_player_Down:

ret

;;FUNCIONES DE ANIMACION PARA EL ENEMIGO
animation_enemy_Up::
ld a, e_anim_cd(ix)
cp #0
jr nz, fin_animation_enemy_Up
reset_cd


ld a, e_animation(ix)
cp #0 ;; Comparamos con registro a
jr nz, animation_enemy_Up_02 ;; Z si es igual y NZ si no es igual

  animation_enemy_Up_01:
    ld hl, #_enemy_sp_01 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==1
      jr fin_animation_enemy_Up 

    animation_enemy_Up_02:
        ld a, e_animation(ix)
      cp #1 ;; Comparamos con registro a
      jr nz, animation_enemy_Up_03 ;; Z si es igual y nz si no es igual

      ld hl, #_enemy_sp_02 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==2
      jr fin_animation_enemy_Up ;;terminamos

    animation_enemy_Up_03:
      ld hl, #_enemy_sp_03 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_reset

    fin_animation_enemy_Up:
      dec_cd
ret

animation_enemy_Left::
;ld a, e_anim_cd(ix)
;cp #0                                                       AL DESCOMENTAR PETA HACIA LA IZQ
;jr nz, fin_animation_enemy_Left
;reset_cd

ld a, e_animation(ix)
cp #0 ;; Comparamos con registro a
jr nz, animation_enemy_Left_02 ;; Z si es igual y NZ si no es igual

  animation_enemy_Left_01:
    ld hl, #_enemy_sp_05 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==1
      jr fin_animation_enemy_Left 

    animation_enemy_Left_02:
        ld a, e_animation(ix)
      cp #1 ;; Comparamos con registro a
      jr nz, animation_enemy_Left_03 ;; Z si es igual y nz si no es igual

      ld hl, #_enemy_sp_06 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==2
      jr fin_animation_enemy_Left ;;terminamos

    animation_enemy_Left_03:
      ld hl, #_enemy_sp_07 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_reset

    fin_animation_enemy_Left:
;     dec_cd
ret


animation_enemy_Right::
ld a, e_anim_cd(ix)
cp #0
jr nz, fin_animation_enemy_Right
reset_cd

ld a, e_animation(ix)
cp #0 ;; Comparamos con registro a
jr nz, animation_enemy_Right_02 ;; Z si es igual y NZ si no es igual

  animation_enemy_Right_01:
    ld hl, #_enemy_sp_09 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==1
      jr fin_animation_enemy_Right 

    animation_enemy_Right_02:
        ld a, e_animation(ix)
      cp #1 ;; Comparamos con registro a
      jr nz, animation_enemy_Right_03 ;; Z si es igual y nz si no es igual

      ld hl, #_enemy_sp_10 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==2
      jr fin_animation_enemy_Right ;;terminamos

    animation_enemy_Right_03:
      ld hl, #_enemy_sp_11 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_reset

    fin_animation_enemy_Right:
      dec_cd
ret

animation_enemy_Down::

ld a, e_anim_cd(ix)
cp #0
jr nz, fin_animation_enemy_Down
reset_cd


ld a, e_animation(ix)
cp #0 ;; Comparamos con registro a
jr nz, animation_enemy_Down_02 ;; Z si es igual y NZ si no es igual

  animation_enemy_Down_01:
    ld hl, #_enemy_sp_13 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==1
      jr fin_animation_enemy_Down 

    animation_enemy_Down_02:
        ld a, e_animation(ix)
      cp #1 ;; Comparamos con registro a
      jr nz, animation_enemy_Down_03 ;; Z si es igual y nz si no es igual

      ld hl, #_enemy_sp_14 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_plus ;;terminamos con counter_plus==2
      jr fin_animation_enemy_Down ;;terminamos

    animation_enemy_Down_03:
      ld hl, #_enemy_sp_15 ;Cargamos su sprite correspondiente
      ld e_sp_h(ix), h
      ld e_sp_l(ix), l

      counter_reset

    fin_animation_enemy_Down:
      dec_cd
ret


;;FUNCIONES DE ANIMACION PARA EL FUERTE

animation_Fort_ix::
  
  ld a, e_hp(ix)
  cp #5
  jr z, end_anim_fort_ix

  life_4_ix:
  cp #4
  jr nz, life_3_ix

  ld hl, #_base_enemy_sp_1 ;Cargamos su sprite correspondiente
  ld e_sp_h(ix), h
  ld e_sp_l(ix), l

  jr end_anim_fort_ix

  life_3_ix:
  cp #3
  jr nz, life_2_ix

  ld hl, #_base_enemy_sp_2 ;Cargamos su sprite correspondiente
  ld e_sp_h(ix), h
  ld e_sp_l(ix), l

  jr end_anim_fort_ix

  life_2_ix:
  cp #2
  jr nz, life_1_ix

  ld hl, #_base_enemy_sp_3 ;Cargamos su sprite correspondiente
  ld e_sp_h(ix), h
  ld e_sp_l(ix), l

  jr end_anim_fort_ix

  life_1_ix:
  ld hl, #_base_enemy_sp_4 ;Cargamos su sprite correspondiente
  ld e_sp_h(ix), h
  ld e_sp_l(ix), l

  end_anim_fort_ix:
  call  render_fort_IX

ret

animation_Fort_iy::
  
  ld a, e_hp(iy)
  cp #5
  jr z, end_anim_fort_iy

  life_4_iy:
  cp #4
  jr nz, life_3_iy

  ld hl, #_base_enemy_sp_1 ;Cargamos su sprite correspondiente
  ld e_sp_h(iy), h
  ld e_sp_l(iy), l

  jr end_anim_fort_iy

  life_3_iy:
  cp #3
  jr nz, life_2_iy

  ld hl, #_base_enemy_sp_2 ;Cargamos su sprite correspondiente
  ld e_sp_h(iy), h
  ld e_sp_l(iy), l

  jr end_anim_fort_iy

  life_2_iy:
  cp #2
  jr nz, life_1_iy

  ld hl, #_base_enemy_sp_3 ;Cargamos su sprite correspondiente
  ld e_sp_h(iy), h
  ld e_sp_l(iy), l

  jr end_anim_fort_iy

  life_1_iy:
  ld hl, #_base_enemy_sp_4 ;Cargamos su sprite correspondiente
  ld e_sp_h(iy), h
  ld e_sp_l(iy), l

  end_anim_fort_iy:
  call  render_fort_IY
ret