;  ----------------------------LICENSE NOTICE-----------------------------
;
;  This file is part of the Hey Sailor! game for Amstrad CPC 464.
;  Copyright (C) 2019  Daniel Bravo García  (dani_jijona@hotmail.com)
;  Copyright (C) 2019  David Navas Díaz   (dnavasdiaz97@gmail.com)
;  Copyright (C) 2019  Juan Manuel Ruiz Culiañez  (pokejuanma@gmail.com)
;
;  Hey Sailor! is free software: you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  Hey Sailor! is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with Hey Sailor!. If not, see <http://www.gnu.org/licenses/>.
;
; ----------------------------------------------------------------------



.include "man/entity_manager.h.s"
.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "sys/animation_system.h.s"
.include "sys/render_system.h.s"
.include "man/game_manager.h.s"
;sys_collision_init::
;	call man_entity_collider_getArrayHL
;	ld	 (_ent_array_ptr), hl
;	ret

.macro saveSetBorder _color
	push hl
	cpctm_setBorder_asm	_color
	pop hl
.endm;

.macro sound_hit_player
		;;AQUI VA EL SONIDO DE RECIBIR DAÑO
  			   ld de, #0x49f
			   call cpct_akp_SFXInit_asm
			   ld l, #1
			   ld h, #15
			   ld e, #25
			   ld d, #0
  			   ld bc, #0x0000
   			   ld a, #100
   			   call cpct_akp_SFXPlay_asm
.endm

sys_collision_update::
  call entityman_getEntityArray_IX	
  ;;comprobamos que hay mas de una entidad
  cp 	#1
  ret z
  ;;si hay mas de una entidad seguimos
  ld b, a
  dec b
  push af

  jr _init_iy


  _next_ix:
    push af
  	ld  de, #sizeof_e
	add ix, de
	ld b, a
	dec b                     ;;Contador emtidades IY
	xor a
	cp b

	jp z, _pop_af

  _init_iy:
	ld__d_ixh
	ld__e_ixl
					;;Cargar en IY, DE --> CARGA IX EN IY
    ld__iyl_e
    ld__iyh_d

  _next_iy:
    ld  de, #sizeof_e
	add iy, de
push bc
  call sys_collision_check







		jp   c, _end;



		_collision:
			ld 		a, e_type(ix)	
			cp  	#e_type_fort	;;Comprobacion para ver si hay que renderizar el fuerte
			call  	z, fort_needs_render
			
			ld a, e_type(ix)	;;Cargamos el tipo de la primera entidad;

			cp  #e_type_player	;;Comparamos con el player
			jr z, _first_player ;;Es el player;

			cp  #e_type_enemy   ;;Comparamos con un enemy
			jr z, _first_enemy  ;;Es un enemy;

			cp  #e_type_fort	;;Comparamos con un fuerte
			jr z, _first_enemy ;;Es un fuerte;

			cp  #e_type_bullet	 ;;Comparamos con la bala
			jr z, _first_bullet  ;;Es bala;

			cp #e_type_enemy_bullet ;;Comparamos con una bala enemiga
			jr z, _first_enemy_bullet 

			jp _end				 ;;Si no es nada salimos;

			_first_player:
				ld a, e_type(iy)        ;;Cargamos el tipo de la otra entidad
				cp #e_type_enemy        ;;Comprueba si la colision es con enemigo
				jr z, _lose_hp          ;;Si lo es pierden vida ambos
				cp #e_type_fort         ;;Comprueba si la colision es con enemigo
				jr z, _lose_hp          ;;Si lo es pierden vida ambos
				cp #e_type_enemy_bullet 
				jr z, _lose_hp
				jp _end                 ;;Si no lo es ignoramos la colision;

			_first_enemy:
				ld a, e_type(iy)         ;;Cargamos el tipo de la otra entidad
				cp #e_type_player        ;;Comprueba si la colision es con el player
				jr z, _lose_hp           ;;Si lo es pierden vida ambos
				cp #e_type_bullet        ;;Si no comprueba si es con la bala
				jr z, _lose_hp           ;;Si lo es pierden vida ambos
				jp _end			         ;;Si no es ninguna, ignoramos colision;

			_first_bullet:
				ld a, e_type(iy)   ;;Cargamos el tipo de la otra entidad
				cp #e_type_enemy   ;;Comprueba si ha chocado con un enemigo
				jr z, _lose_hp     ;;Si lo hace, pierden vida ambos
				cp #e_type_enemy_bullet ;;Es una bala enemiga
				jr z, _lose_hp
				cp #e_type_fort
				jr z, _lose_hp
				jp _end		   	   ;;Si no es ninguna, ignoramos colision;

			_first_enemy_bullet:
				ld a, e_type(iy)   ;;Cargamos el tipo de la otra entidad
				cp #e_type_player  ;;Comprueba si ha chocado con el player
				jr z, _lose_hp     ;;Si lo hace, pierden vida ambos
				cp #e_type_bullet 
				jr z, _lose_hp
				jp _end		   	   ;;Si no es ninguna, ignoramos colision;

			_lose_hp:       	   ;;Pierden 1 vida ambos
				
				ld a, e_hp(ix)
				cp #0
				jr z, _next;

				dec a
				ld e_hp(ix), a;

				ld a, e_type(ix)

				cp #e_type_fort ;;Comprobamos si se trata de un fuerte para cambiar su sprite
				call z, animation_Fort_ix

				_next:
				ld a, e_hp(iy)
				cp #0
				jr z, _move_player
				dec a
				ld e_hp(iy), a;

				
				_move_player:
				ld a, e_type(ix)
				cp #0
				jr nz, _no_ix
				ld e_x(ix), #37
				ld e_y(ix), #150
				call change_lives_sprites
				call hurt_pirate
				call game_man_activate_cooldown
				sound_hit_player
				jr _end;

				_no_ix:
				ld a, e_type(iy)
				cp #0
				jr nz, _end
				ld e_x(iy), #37
				ld e_y(iy), #150;

				cp #e_type_fort ;;Comprobamos si se trata de un fuerte para cambiar su sprite
				call z, animation_Fort_iy
				call change_lives_sprites
				call hurt_pirate
				call game_man_activate_cooldown
				sound_hit_player

				
				jr _end




  _end:
  pop bc
	  dec b                     ;;Contador emtidades IY
	  xor a
	  cp b

	  jp nz, _next_iy

  _pop_af:
  pop  af


  dec a  					;;Contador entidades IX
  cp  #0
  ret z 

  jp  _next_ix


ret

;sys_collision_update::

;	saveSetBorder HW_GREEN
;	_next_ix:
;		;;Puntero a la primera entidad
;		ld	e, (hl)
;		inc hl
;		ld	d, (hl)
;		inc hl;

;		;if DE = null -> salir
;		ld 	a, e
;		or 	d
;		jp  z, _exit
;		;******************/;

;		push hl;

;		;;Lo meto en IX
;		ld__ixl_e
;		ld__ixh_d;

;	_next_iy:
;		;;****************************
;		;;Puntero a la segunda entidad
;		ld e, (hl)
;		inc hl
;		ld  d, (hl)
;		inc hl;

;		;if DE = null -> salir
;		ld 	a, e
;		or 	d
;		jp  z, _pop
;		;******************/;

;		;;Lo meto en IY
;		ld__iyl_e
;		ld__iyh_d
;		;;****************************;

;		call sys_collision_check ;;Compruebo la colision
;		jr   c, _no_collision;

;		_collision:
;			ld a, e_type(ix)	;;Cargamos el tipo de la primera entidad;

;			cp  #e_type_player	;;Comparamos con el player
;			jr z, _first_player ;;Es el player;

;			cp  #e_type_enemy   ;;Comparamos con un enemy
;			jr z, _first_enemy  ;;Es un enemy;

;			cp  #e_type_fort	;;Comparamos con un fuerte
;			jr z, _first_enemy ;;Es un fuerte;

;			cp  #e_type_bullet	 ;;Comparamos con la bala
;			jr z, _first_bullet  ;;Es bala;

;			ret z				 ;;Si no es nada salimos;

;			_first_player:
;				ld a, e_type(iy)   ;;Cargamos el tipo de la otra entidad
;				cp #e_type_enemy   ;;Comprueba si la colision es con enemigo
;				jr z, _lose_hp     ;;Si lo es pierden vida ambos
;				cp #e_type_fort   ;;Comprueba si la colision es con enemigo
;				jr z, _lose_hp     ;;Si lo es pierden vida ambos
;				jr _end            ;;Si no lo es ignoramos la colision;

;			_first_enemy:
;				ld a, e_type(iy)   ;;Cargamos el tipo de la otra entidad
;				cp #e_type_player  ;;Comprueba si la colision es con el player
;				jr z, _lose_hp     ;;Si lo es pierden vida ambos
;				cp #e_type_bullet  ;;Si no comprueba si es con la bala
;				jr z, _lose_hp     ;;Si lo es pierden vida ambos
;				jr _end			   ;;Si no es ninguna, ignoramos colision;

;			_first_bullet:
;				ld a, e_type(iy)   ;;Cargamos el tipo de la otra entidad
;				cp #e_type_enemy   ;;Comprueba si ha chocado con un enemigo
;				jr z, _lose_hp     ;;Si lo hace, pierden vida ambos
;				jr _end		   	   ;;Si no es ninguna, ignoramos colision;

;			_lose_hp:       	   ;;Pierden 1 vida ambos
;				ld a, e_hp(ix)
;				cp #0
;				jr z, _next;

;				dec a
;				ld e_hp(ix), a;

;				_next:
;				ld a, e_hp(iy)
;				cp #0
;				jr z, _move_player
;				dec a
;				ld e_hp(iy), a;

;				
;				_move_player:
;				ld a, e_type(ix)
;				cp #0
;				jr nz, _no_ix
;				ld e_x(ix), #30
;				ld e_y(ix), #10
;				jr _end;

;				_no_ix:
;				ld a, e_type(iy)
;				cp #0
;				jr nz, _end
;				ld e_x(iy), #30
;				ld e_y(iy), #10;

;				jr _end;

;			_end:
;			saveSetBorder HW_RED
;			jp	_next_iy;

;		_no_collision:
;			saveSetBorder HW_GREEN
;			jp  _next_iy;
;

;	_pop:
;		pop hl
;		jp  _next_ix
;	_exit:
;		saveSetBorder HW_WHITE
;		ret

;;********************************
;; SYS COLLISION CHECK
;; INPUT: Pointer al primer elemento con colision
;; DESTROYS: AF
;; RETURNS Carry: (1) No colision, (0) colision
sys_collision_check::
	;; EJE X
	ld 	a, e_x(ix)		;A = e_x(ix)
	add e_w(ix)			;A = e_x(ix) + e_w(ix)
	sub e_x(iy)			;A = e_x(ix) + e_w(ix) - e_x(iy)
	ret c

	ld 	a, e_x(iy)		;A = e_x(ix)
	add e_w(iy)			;A = e_x(ix) + e_w(ix)
	sub e_x(ix)			;A = e_x(ix) + e_w(ix) - e_x(iy)
	ret c

	;;EJE Y
	ld 	a, e_y(ix)		;A = e_x(ix)
	add e_h(ix)			;A = e_x(ix) + e_w(ix)
	sub e_y(iy)			;A = e_x(ix) + e_w(ix) - e_x(iy)
	ret c

	ld 	a, e_y(iy)		;A = e_x(ix)
	add e_h(iy)			;A = e_x(ix) + e_w(ix)
	sub e_y(ix)			;A = e_x(ix) + e_w(ix) - e_x(iy)
	ret

