;  ----------------------------LICENSE NOTICE-----------------------------
;
;  This file is part of the Hey Sailor! game for Amstrad CPC 464.
;  Copyright (C) 2019  Daniel Bravo García  (dani_jijona@hotmail.com)
;  Copyright (C) 2019  David Navas Díaz   (dnavasdiaz97@gmail.com)
;  Copyright (C) 2019  Juan Manuel Ruiz Culiañez  (pokejuanma@gmail.com)
;
;  Hey Sailor! is free software: you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  Hey Sailor! is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with Hey Sailor!. If not, see <http://www.gnu.org/licenses/>.
;
; ----------------------------------------------------------------------



;;
;; Input System
;;

.include "man/entity_manager.h.s"
.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "assets/assets.h.s"
.include "man/game_manager.h.s"
.include "sys/render_system.h.s"
.include "sys/animation_system.h.s"
.include "assets/music/music.h.s"

;; Includes para los mapas comprimidos
.include "src/assets/compressed/mimapa.h.s"
.include "src/assets/compressed/mifotoinicio.h.s"
.include "src/assets/compressed/mifotopausa.h.s"
.include "src/assets/compressed/mifotogameover.h.s"
.include "src/assets/compressed/mifotonextstage.h.s"
.include "src/assets/compressed/mifotonextstage2.h.s"
.include "src/assets/compressed/mifotoyouwin.h.s"

_cooldown: .db 35
_stage_clear_counter: .db 2

sys_input_update::
  call ent_movekeyboard
  call ent_movejoystick
ret

input_screen::
  ;;SUPER IMPORTANTE NO ALTERAR EL ORDEN 
 input_screen_loop:
  call cpct_waitVSYNC_asm
  call cpct_waitVSYNC_asm
  call cpct_akp_musicPlay_asm

  call  cpct_scanKeyboard_f_asm
  ld  hl, #Key_Return
  call  cpct_isKeyPressed_asm
  ;call  cpct_isAnyKeyPressed_asm
  jr nz, end_input_screen
  
input_screen_joystick_loop:
  call  cpct_scanKeyboard_f_asm
  ld hl, #Joy0_Fire1
  call  cpct_isKeyPressed_asm
  ;call  cpct_isAnyKeyPressed_asm
  jr z, input_screen_loop

end_input_screen:
  call cpct_akp_stop_asm

ret

pause_screen::

  ;;AQUI VA EL SONIDO DE LA PAUSA
   ld de, #0x451
   call cpct_akp_SFXInit_asm
   ld l, #1
   ld h, #15
   ld e, #85
   ld d, #0
   ld bc, #0x0000
   ld a, #100
   call cpct_akp_SFXPlay_asm

pause_screen_loop:
  call cpct_akp_stop_asm
  call  cpct_scanKeyboard_f_asm
  ld  hl, #Key_Return
  call  cpct_isKeyPressed_asm
  jr nz, end_pause_screen

pause_screen_joystick_loop:
  call cpct_akp_stop_asm
  call  cpct_scanKeyboard_f_asm
  ld hl, #Joy0_Fire1
  call  cpct_isKeyPressed_asm
  jr z, pause_screen_loop

end_pause_screen:
  ;;Una vez pulsado el intro resutaramos el fondo y seguimos jugando
  ld hl,  #_mimapa_end
  call rendersys_set_background

  call change_lives_sprites ;; Restauramos la interfaz
  ld  a, (_level)
  call render_num_level
  call render_face_sailor
  call all_forts_needs_render
ret

stage_clear_screen::
  call cpct_akp_stop_asm;;Paramos la musica de antes

  ld a, (_stage_clear_counter)
  cp #0
  jr z, stage_clear_extra_life
  ;; Codigo para la descompresion de backgrounds
   ld hl,  #_mifotonextstage_end

  ld a, (_stage_clear_counter)
  dec a
  ld (_stage_clear_counter), a

  jr stage_clear_standard

  stage_clear_extra_life:
   ld hl,  #_mifotonextstage2_end
   ld a, #2
   ld (_stage_clear_counter), a

   stage_clear_standard:
   call rendersys_set_background

   ;;Ponemos la musica de la pantalla de muerte
    ld de, #_stage_clear_song
    call cpct_akp_musicInit_asm

   stage_clear_loop:
  ;;SUPER IMPORTANTE NO ALTERAR EL ORDEN 
  call cpct_waitVSYNC_asm
  call cpct_waitVSYNC_asm
  call cpct_akp_musicPlay_asm

  call  cpct_scanKeyboard_f_asm
  ld  hl, #Key_Return
  call  cpct_isKeyPressed_asm
  jr nz, end_stage_clear_screen

  stage_clear_joystick_loop:
  call  cpct_scanKeyboard_f_asm
  ld hl, #Joy0_Fire1
  call  cpct_isKeyPressed_asm
  jr z, stage_clear_loop

  end_stage_clear_screen:
  call cpct_akp_stop_asm

   ;;Ponemos la musica del juego principal
  ld de, #_main_song
  call cpct_akp_musicInit_asm
ret


you_win_screen::
  ld a, #2
  ld (_stage_clear_counter),a

  call cpct_akp_stop_asm;;Paramos la musica de antes
  ;; Codigo para la descompresion de backgrounds
   ld hl,  #_mifotoyouwin_end
   call rendersys_set_background

   ;;Ponemos la musica de la pantalla de muerte
    ld de, #_victory_song
    call cpct_akp_musicInit_asm

   you_win_loop:
  ;;SUPER IMPORTANTE NO ALTERAR EL ORDEN 
  call cpct_waitVSYNC_asm
  call cpct_waitVSYNC_asm
  call cpct_akp_musicPlay_asm

  call  cpct_scanKeyboard_f_asm
  ld  hl, #Key_Return
  call  cpct_isKeyPressed_asm
  jr nz, end_you_win_screen

  you_win_joystick_loop:
  call  cpct_scanKeyboard_f_asm
  ld hl, #Joy0_Fire1
  call  cpct_isKeyPressed_asm
  jr z, you_win_loop

  end_you_win_screen:
  call cpct_akp_stop_asm

   ;;Ponemos la musica del juego principal
  ld de, #_main_song
  call cpct_akp_musicInit_asm
ret    ;; dara esto algun problema porque vuelve a donde se le ha llamado?


gameOver_screen::
  ld a, #2
  ld (_stage_clear_counter),a
  
  call cpct_akp_stop_asm;;Paramos la musica de antes
  ;; Codigo para la descompresion de backgrounds
   ld hl,  #_mifotogameover_end
   call rendersys_set_background

   ;;Ponemos la musica de la pantalla de muerte
    ld de, #_death_song
    call cpct_akp_musicInit_asm

   game_over_loop:
  ;;SUPER IMPORTANTE NO ALTERAR EL ORDEN 
  call cpct_waitVSYNC_asm
  call cpct_waitVSYNC_asm
  call cpct_akp_musicPlay_asm

  call  cpct_scanKeyboard_f_asm
  ld  hl, #Key_Return
  call  cpct_isKeyPressed_asm
  jr nz, end_game_over_screen

  game_over_joystick_loop:
  call  cpct_scanKeyboard_f_asm
  ld hl, #Joy0_Fire1
  call  cpct_isKeyPressed_asm
  jr z, game_over_loop

  end_game_over_screen:
  call cpct_akp_stop_asm

  ;;Ponemos la musica del juego principal
  ld de, #_main_song
  call cpct_akp_musicInit_asm

  call game_man_restart
ret    ;; dara esto algun problema porque vuelve a donde se le ha llamado?



ent_movekeyboard::
  ;; Escaneamos el teclado
  call cpct_scanKeyboard_f_asm

  ;; Comprobamos si las teclas han sido pulsadas
  ld hl, #Key_O
  call cpct_isKeyPressed_asm
  jr z, O_NotPressed

O_Pressed:
  
  ld e_vx(ix), #-1 ;Originalmente aqui habia un -2, pero lo dejamos en 1 para que el sprite pinte bien
  ld e_vy(ix), #0

O_NotPressed:
  ld hl, #Key_P
  call cpct_isKeyPressed_asm
  jr z, P_NotPressed

P_Pressed:
  
  ld e_vx(ix), #1 ;Originalmente aqui habia un 2, pero lo dejamos en 1 para que el sprite pinte bien
  ld e_vy(ix), #0

P_NotPressed:
  ld hl, #Key_Q
  call cpct_isKeyPressed_asm
  jr z, Q_NotPressed

Q_Pressed:

  ld e_vy(ix), #-2 ;Originalmente aqui habia un -4, pero lo dejamos en 1 para que el sprite pinte bien
  ld e_vx(ix), #0

Q_NotPressed:
  ld hl, #Key_A
  call cpct_isKeyPressed_asm
  jr z, A_NotPressed

A_Pressed:

  ld e_vy(ix), #2 ;Originalmente aqui habia un 4, pero lo dejamos en 1 para que el sprite pinte bien
  ld e_vx(ix), #0
  
A_NotPressed:
  ld hl, #Key_Space
  call cpct_isKeyPressed_asm
  jr z, Space_NotPressed

Space_Pressed:
  ;;Antes comprobamos si podemos disparar por el cooldown
  xor   a     ;;a=0
  ld    iy, #_cooldown
  cp    a, 0(iy) ;;vemos si el contador esta a 0
  jr    nz, no_disparo

disparo:
  ;; DISPARO DANI
  ;;tengo que crear una bala con las velocidades correspondientes
  ld    bc, #0 ;; pongo bc a 0001, para que dispare en el eje x si no te mueves
                ;;esto es provisional, cambiar jugabilidad?


  ;xor   a       ;; a = 0 A ya es 0
  cp    a, e_vx(ix)   ;; comparo con su velocidad para ver hacia donde se dirige
  jr z, no_x
si_x:
  ;; la velocidad del disparo es superior a la del barco
  ld    a, e_vx(ix)
  add   a, a   ;; vel x2
  ld    b, a  ;; ponemos a 1 la velocidad en x de la bala
no_x:

  ;; comprobamos la Y
  cp    a, e_vy(ix)   ;; comparo con su velocidad para ver hacia donde se dirige
  jr z, no_y
si_y:
  ld    a, e_vy(ix)
  add   a, a   ;; vel x2
  ld    c, a  ;; ponemos a 1 la velocidad en x de la bala
no_y:

  ld    h, e_x(ix)
  ld    l, e_y(ix)
  call create_bullet    ;; le paso en B la velocidad x y en C la velocidad y
                        ;; tambien en hl la posicion x y
  ;; ahora creo otra bala que va en sentido contrario
  ;ld    h, e_x(ix)
  ;ld    l, e_y(ix)
  ;ld    b, a
  ;neg
  ;ld    a, b
;
;  ;ld    c, a
;  ;neg
  ;ld    a, c
  ;call create_bullet      ;; create bullet esta en el entity manager

  ;; Reiniciamos el contador de cooldown
  ld    a, #35
  ld    (_cooldown), a

  
  ;;AQUI VA EL SONIDO DEL DISPARO
  ld de, #0x451
  call cpct_akp_SFXInit_asm
   ld l, #1
   ld h, #15
   ld e, #55
   ld d, #0
   ld bc, #0x0000
   ld a, #100
   call cpct_akp_SFXPlay_asm




Space_NotPressed:
  
  ld hl, #Key_Esc
  call cpct_isKeyPressed_asm
  jr z, Escape_NotPressed

Escape_Pressed:
   call cpct_akp_stop_asm
;;Ponemos imagen de pausa
  ld hl,  #_mifotopausa_end
  call rendersys_set_background

  call pause_screen  ;; LLAMAMOS A PAUSA
  
Escape_NotPressed:



no_disparo:
;; cooldown para el disparo, para que no dispares muchas veces al pulsar 
  ;; 1 vez la tecla
  xor   a
  ld    iy, #_cooldown
  cp    a, 0(iy) ;;vemos si el contador esta a 0
  jr    z, es_0

no_es_0:
  ld    a, (_cooldown)
  dec   a
  ld    (_cooldown), a
es_0:

  ret


;;MAPA JOYSTICK
  ;;Joystick arriba: Joy0_Up 
  ;;Joystick abajo:  Joy0_Down
  ;;Joystick izquierda: Joy0_Left -- Trabajando
  ;;Joystick derecha:   Joy0_Right

  ;;Boton Joystick pausa:: Tecla escape del teclado
  ;;Boton start game:: Joy0 ???
  ;;Boton disparo:: Joy0_Fire1
ent_movejoystick::
  ;; Escaneamos el teclado
  call cpct_scanKeyboard_f_asm

  ;; Comprobamos si las teclas han sido pulsadas
  ld hl, #Joy0_Left ;;Originalmente era #key_O en el teclado
  call cpct_isKeyPressed_asm
  jr z, Joy0_Left_NotPressed

Joy0_Left_Pressed: ;;IZQUIERDA
  ld e_vx(ix), #-1 ;Originalmente aqui habia un -2, pero lo dejamos en 1 para que el sprite pinte bien
  ld e_vy(ix), #0

Joy0_Left_NotPressed: ;;IZQUIERDA
  ld hl, #Joy0_Right ;;Originalmente era #key_P en el teclado
  call cpct_isKeyPressed_asm
  jr z, Joy0_Right_NotPressed

Joy0_Right_Pressed: ;;DERECHA
  ld e_vx(ix), #1 ;Originalmente aqui habia un 2, pero lo dejamos en 1 para que el sprite pinte bien
  ld e_vy(ix), #0

Joy0_Right_NotPressed: ;;DERECHA
  ld hl, #Joy0_Up ;;Originalmente era #key_Q en el teclado
  call cpct_isKeyPressed_asm
  jr z, Joy0_Up_NotPressed

Joy0_Up_Pressed: ;;ARRIBA
  ld e_vy(ix), #-2 ;Originalmente aqui habia un -4, pero lo dejamos en 2 para que el sprite pinte bien
  ld e_vx(ix), #0

Joy0_Up_NotPressed: ;;ARRIBA
  ld hl, #Joy0_Down ;;Originalmente era #key_A en el teclado
  call cpct_isKeyPressed_asm
  jr z, Joy0_Down_NotPressed

Joy0_Down_Pressed: ;;ABAJO
  ld e_vy(ix), #2 ;Originalmente aqui habia un 4, pero lo dejamos en 2 para que el sprite pinte bien
  ld e_vx(ix), #0
  
Joy0_Down_NotPressed: ;;ABAJO
  ld hl, #Joy0_Fire1  ;;Originalmente era #key_Space en el teclado
  call cpct_isKeyPressed_asm
  jr z, Joy0_Fire1_NotPressed

Joy0_Fire1_Pressed:
  ;;Antes comprobamos si podemos disparar por el cooldown
  xor   a     ;;a=0
  ld    iy, #_cooldown
  cp    a, 0(iy) ;;vemos si el contador esta a 0
  jr    nz, no_disparo_Joystick

disparo_Joystick:
  ;; DISPARO DANI
  ;;tengo que crear una bala con las velocidades correspondientes
  ld    bc, #0 ;; pongo bc a 0001, para que dispare en el eje x si no te mueves
                ;;esto es provisional, cambiar jugabilidad?


  ;xor   a       ;; a = 0 A ya es 0
  cp    a, e_vx(ix)   ;; comparo con su velocidad para ver hacia donde se dirige
  jr z, no_x_Joystick
si_x_Joystick:
  ;; la velocidad del disparo es superior a la del barco
  ld    a, e_vx(ix)
  add   a, a   ;; vel x2
  ld    b, a  ;; ponemos a 1 la velocidad en x de la bala
no_x_Joystick:

  ;; comprobamos la Y
  cp    a, e_vy(ix)   ;; comparo con su velocidad para ver hacia donde se dirige
  jr z, no_y_Joystick
si_y_Joystick:
  ld    a, e_vy(ix)
  add   a, a   ;; vel x2
  ld    c, a  ;; ponemos a 1 la velocidad en x de la bala
no_y_Joystick:

  ld    h, e_x(ix)
  ld    l, e_y(ix)
  call create_bullet    ;; le paso en B la velocidad x y en C la velocidad y
                        ;; tambien en hl la posicion x y
  ;; ahora creo otra bala que va en sentido contrario
  ;ld    h, e_x(ix)
  ;ld    l, e_y(ix)
  ;ld    b, a
  ;neg
  ;ld    a, b
;
;  ;ld    c, a
;  ;neg
  ;ld    a, c
  ;call create_bullet      ;; create bullet esta en el entity manager

  ;; Reiniciamos el contador de cooldown
  ld    a, #35
  ld    (_cooldown), a


  ;;AQUI VA EL SONIDO DEL DISPARO
  ld de, #0x451
  call cpct_akp_SFXInit_asm
   ld l, #1
   ld h, #15
   ld e, #55
   ld d, #0
   ld bc, #0x0000
   ld a, #100
   call cpct_akp_SFXPlay_asm

Joy0_Fire1_NotPressed:
  
  ld hl, #Key_Esc ;;Tecla escape para pausa
  call cpct_isKeyPressed_asm
  jr z, Escape_NotPressed_Joystick

Escape_Pressed_Joystick:
;;Ponemos imagen de pausa
  ld hl,  #_mifotopausa_end
  call rendersys_set_background
  call pause_screen  ;; LLAMAMOS A PAUSA
  
Escape_NotPressed_Joystick:

no_disparo_Joystick:
;; cooldown para el disparo, para que no dispares muchas veces al pulsar 
  ;; 1 vez la tecla
  xor   a
  ld    iy, #_cooldown
  cp    a, 0(iy) ;;vemos si el contador esta a 0
  jr    z, es_0_Joystick

no_es_0_Joystick:
  ld    a, (_cooldown)
  dec   a
  ld    (_cooldown), a
es_0_Joystick:
  
  ret