;  ----------------------------LICENSE NOTICE-----------------------------
;
;  This file is part of the Hey Sailor! game for Amstrad CPC 464.
;  Copyright (C) 2019  Daniel Bravo García  (dani_jijona@hotmail.com)
;  Copyright (C) 2019  David Navas Díaz   (dnavasdiaz97@gmail.com)
;  Copyright (C) 2019  Juan Manuel Ruiz Culiañez  (pokejuanma@gmail.com)
;
;  Hey Sailor! is free software: you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  Hey Sailor! is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with Hey Sailor!. If not, see <http://www.gnu.org/licenses/>.
;
; ----------------------------------------------------------------------



;; Physics system

.include "man/entity_manager.h.s"
.include "cpctelera.h.s"

;; Ponemos las dimensiones de pantalla con las que trabajaremos
screen_width    = 80
screen_height   = 200
screen_height_r   = 166
pixel_0			= 34


physicsys_init::
;;TODO
ret


physicsys_update::
call entityman_getNumEntities_A
ld 	b, a 		;;load in B number of entities

	_update_loop:

		;; UPDATE X
		ld 	a, #screen_width  	;; (Screen Width)
		sub	e_w(ix)			;; |
		ld 	c, a 				;; C = Screen.Width - Entity.Width 

		ld 	a, e_x(ix)			;; |
		add 	e_vx(ix)			;; A = E_X + E_VX (Add Force)
		cp 	c 				;; |
		jr 	nc, invalid_x 		;; if A == 0 or 1 Invalid Position

		valid_x:
			ld 	e_x(ix), a 		;; Update Position X
			jr 	endif_x 		;; ...
		invalid_x:  ;;Comentado para que las balas se destruyan y no reboten al tocar pared
			;ld 	a, e_vx(ix) 	;; |
			;neg				;; e_vx*-1 (Reverse Velocity)
			;ld 	e_vx(ix),a 		;; Update e_vx
			;jr 	endif_x		;; ...
		endif_x:

		;; UPDATE Y
		ld 	a, #screen_height 	;;(Screen Height)
		sub 	e_h(ix)			;; |
		ld 	c, a 				;; C = Screen.Height - Entity.Height

		ld 	a, e_y(ix)			;; |
		add 	e_vy(ix) 			;; Add Fore Y
		cp 	c 				;; | 
		jr 	nc, invalid_y 		;; if not carry, (0 or more) invalid position

		valid_y:
			ld 	e_y(ix), a 		;; Update Position Y
			jr 	endif_y		;; ...
		invalid_y: ;;Comentado para que las balas se destruyan y no reboten al tocar pared
			;ld 	a, e_vy(ix)		;; |
			;neg 				;; e_vy *= -1
			;ld 	e_vy(ix),a 		;; Update Velocity
		endif_y:
			dec 	b 			;; num_entities--
			ret 	z			;; return if  num_entities = 0

			ld 	de, #sizeof_e 	;; DE = Entity_size
			add 	ix, de 		;; IX, next entity
	jr 	_update_loop

ret

;;DESTRUYE LA BALA QUE BALA A TOCAR EL BORDE
;; NO TIENE INPUTS
physicsys_destroy_bullets_update::
	call entityman_getBulletArray_IX
	call entityman_getNumBullets_A
	ld 	b, a 		;;load in B number of entities
	;; En IX tendre el array de balas y en IY su entidad

	;;comprobamos si hay 0 balas
	xor 	a	;;a=0
	cp	a, b
	ret 	z

	_bullets_loop:
		;; cargamos los datos de la bala en iy
		ld	e, ptr_bullet_l(ix)
		ld	d, ptr_bullet_h(ix)
		ld__iyh_d
		ld__iyl_e

		;; UPDATE X
		ld 	a, #screen_width 		;; (Screen Width)
		sub	e_w(iy)			;; |
		ld 	c, a 				;; C = Screen.Width - Entity.Width 

		ld 	a, e_x(iy)			;; |
		add 	e_vx(iy)			;; A = E_X + E_VX (Add Force)
		cp 	c 				;; |
		jr 	nc, invalid_x_bullets 		;; if A == 0 or 1 Invalid Position

		ld 	c, #1		;; Start screen width

		ld 	a, e_x(iy)			;; |
		add 	e_vx(iy) 			;; Add Fore X
		;; Comprobamos si se sale por arriba
		;; c = 0
		cp	c
		jr	z, invalid_x_bullets	;; if A == C 
		jr	c, invalid_x_bullets	;; if A < C	 esto hace if A<=C

		valid_x_bullets:
			;; si la posicion en x es valida, comprobamos en y
			;; COMPROBAMOS Y
			ld 	a, #screen_height-2		;;(Screen Height)
			sub 	e_h(iy)			;; |
			ld 	c, a 				;; C = Screen.Height - Entity.Height

			ld 	a, e_y(iy)			;; |
			add 	e_vy(iy) 			;; Add Fore Y
			cp 	c 				;; | 
			jr 	nc, invalid_y_bullets 		;; if not carry, (0 or more) invalid position


			ld 	c, #34		;;(Screen Height)
			ld 	a, e_y(iy)			;; |
			add 	e_vy(iy) 			;; Add Fore Y

			;; Comprobamos si se sale por arriba
			;; c = 0
			cp	c
			jr	z, invalid_y_bullets	;; if A == C 
			jr	c, invalid_y_bullets	;; if A < C	 esto hace if A<=C

			jr	endif_balas		;; si esta todo bien se salta lo de borrar
		invalid_x_bullets:
		invalid_y_bullets:
			;cargamos en hl el puntero a la bala
			ld__a_ixl
			ld	l, a
			ld__a_ixh
			ld	h, a
			call entityman_destroy_bullet ;; destruimos la bala


		endif_balas:
			;;comprobamos si hay 0 balas
			xor 	a	;;a=0
			cp	a, b
			ret 	z
			dec 	b 			;; num_bullets--
			ret 	z			;; return if  num_bullets = 0

			ld 	de, #sizeof_ptr_to_bullet 	;; DE = sizeof_ptr_to_bullet
			add 	ix, de 		;; IX, next bullet
	jr 	_bullets_loop
ret

;; MUEVE AL JUGADOR AL OTRO LADO DE LA PANTALLA CUANDO LA TOCA
;;  NO TIENE INPUTS
physicsys_teleport::
	call entityman_getEntityArray_IX	;; player IX
	;; ahora que tenemos el jugador, vemos si va a tocar la pared

	;; COMPROBAMOS X
	ld 	a, #screen_width  	;; (Screen Width)
	sub	e_w(ix)			;; |
	ld 	c, a 				;; C = Screen.Width - Entity.Width 

	ld 	a, e_x(ix)			;; |
	add 	e_vx(ix)			;; A = E_X + E_VX (Add Force)
	cp 	c 				;; |
	jr 	nc, derecha 		;; if A >= C

	;; Comprobamos si se sale por la izquierda
	ld	c, #0			;; c = 0
	cp	c
	jr	z, izquierda	;; if A == C 
	jr	c, izquierda	;; if A < C	 esto hace if A<=C
	jr	endif_x_teleport
derecha:
	sub	a, #screen_width-4	
	ld	e_x(ix), a		;; le quitamos el tamaño de la pantalla
	jr	endif_x_teleport
izquierda:
	ld	a, e_x(ix)
	add	a, #screen_width-6 ;; le resto 5, por la velocidad y uno para que no se meta en el >=
						;; (hay que ajustar este num si se cambia el sprite o velocidad)
	ld	e_x(ix), a		;; le quitamos el tamaño de la pantalla

endif_x_teleport:

	;; COMPROBAMOS Y
	ld 	a, #screen_height-2  	;; (Screen Width)
	sub	e_h(ix)			;; |
	ld 	c, a 				;; C = Screen.Width - Entity.Width 

	ld 	a, e_y(ix)			;; |
	add 	e_vy(ix)			;; A = E_X + E_VX (Add Force)
	cp 	c 				;; |
	jr 	nc, abajo 		;; if A >= C

	;; Comprobamos si se sale por arriba
	ld	c, #34			;; c = 0
	cp	c
	jr	z, arriba	;; if A == C 
	jr	c, arriba	;; if A < C	 esto hace if A<=C
	jr	endif_y_teleport
abajo:
	ld 	a, #35	
	ld	e_y(ix), a		;; le quitamos el tamaño de la pantalla
	jr	endif_y_teleport
arriba:
	ld	a, #screen_height-2
	sub	a, e_h(ix)
	ld	e_y(ix), a		;; le quitamos el tamaño de la pantalla

endif_y_teleport:
ret