;  ----------------------------LICENSE NOTICE-----------------------------
;
;  This file is part of the Hey Sailor! game for Amstrad CPC 464.
;  Copyright (C) 2019  Daniel Bravo García  (dani_jijona@hotmail.com)
;  Copyright (C) 2019  David Navas Díaz   (dnavasdiaz97@gmail.com)
;  Copyright (C) 2019  Juan Manuel Ruiz Culiañez  (pokejuanma@gmail.com)
;
;  Hey Sailor! is free software: you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation, either version 3 of the License, or
;  (at your option) any later version.
;
;  Hey Sailor! is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with Hey Sailor!. If not, see <http://www.gnu.org/licenses/>.
;
; ----------------------------------------------------------------------



;;
;; RENDER SYSTEM
;;

.include "cpct_functions.h.s"
.include "man/entity_manager.h.s"
.include "assets/assets.h.s"
.include "cpctelera.h.s"

; Includes para los mapas comprimidos
.include "src/assets/compressed/mifotoinicio.h.s"

.macro reset_animaiton_pirate_counter

   ld a, #0         
   ld (_pirate_face_animation_counter), a

.endm

_pos_rum_y = 8
_pos_rum1_x = 63
_pos_rum2_x = 68
_pos_rum3_x = 73

_pos_num1_x = 16
_pos_num1_y = 8
_pos_num2_x = 24
_pos_num2_y = 8


_pos_face_pirate_x = 32
_pos_face_pirate_talk_x = 22
_pos_face_pirate_y = 2
_pirate_face_animation_cd   = 50
_pirate_state: .db 0  ;;(0 significa pirata no herido y 1 pirata demacre)

;;Inicialización del sistema de renderizado
;; Cargamos la paleta de 16 bits para el render
rendersys_init:: 
;;TO DO: ARREGLAR ESTA WEA
      ;; Ponemos el modo de video
      ld c, #1 
      call cpct_setVideoMode_asm

      ; Ponemos la paleta del personaje
      ld hl, #_player2_pal
      ld de, #16
      call cpct_setPalette_asm
      cpctm_setBorder_asm HW_PASTEL_YELLOW
ret 

;; Funcion para precalcular la posicion de las entidades 
rendersys_calculate_pos::
   call entityman_getEntityArray_IX

_calculate_pos:
   push af ;; Pila donde guardo a, para que las funciones cpct no se carguen el registro

    ld    a, e_type(ix)    
    cp    #e_type_fort      
    jr    z, _no_calculate

   ;; Pintamos el Sprite NORMALMENTE
   ;; INPUT PARAMETERS:
   ld de, #0xC000 ;; [3] Puntero al inicio de la memoria de video
   ld c, e_x(ix)
   ld b, e_y(ix)
   call cpct_getScreenPtr_asm ;;Calculate video memory location

   ld e_newVP_l(ix), l ;; Guardamos los valores de la nueva posicion
   ld e_newVP_h(ix), h

   _no_calculate:
   pop   af ;; saco a de la pila

   dec a
   ret z

   ld    bc, #sizeof_e
   add   ix, bc
   jr _calculate_pos
ret


;; INPUT:
;;    IX: Pointer to first entity to render
;;    A: Number of entities to render 
rendersys_update::

_renloop:   

  push af ;; Pila donde guardo a, para que las funciones cpct no se carguen el registro

  ;; compruebo que no sea un fuerte    
   ld    a, e_type(ix)     
   cp    #e_type_fort      
   jr    z, _no_render

   ;; IMPORTANTE: ESTE CHACHO DE CODIGO SE ENCARGA DE LIMPIAR EL ANTERIOR DIBUJADO
   ;; LLAMANDO A DRAWSOLIDBOX. NO PONER OTRO METODO CLEAR SIN TENER
   ;; EN CUENTA ESTO PORQUE SE LIA PARDA
   ld   e, e_lastVP_l(ix)
   ld   d, e_lastVP_h(ix)

   ld   a, #0  ;; color
   ld   c, e_w(ix)      ;; Width
   ld   b, e_h(ix)     ;; Height
   call cpct_drawSolidBox_asm ;; [x] --Esta funcion se carga el valor de las variables que usa

   ld e, e_newVP_l(ix)
   ld d, e_newVP_h(ix)
   ld e_lastVP_l(ix), e ;; Guardamos los valores de la posicion anterior
   ld e_lastVP_h(ix), d


   ld   h, e_sp_h(ix)
   ld   l, e_sp_l(ix)   ;;Etiqueta que referencia al codigo del sprite
   
   ld   c, e_w(ix) ;;Ancho de la entidad
   ld   b, e_h(ix) ;;Alto de la entidad

   call cpct_drawSprite_asm

   _no_render:

   pop   af ;; saco a de la pila

   dec a
   ret z

   ld    bc, #sizeof_e
   add   ix, bc
   jr _renloop



;Funcion que descomprime y pone en pantalla lo que referenciemos en hl
;; P. ej; ld hl,  #_mimapa_end
rendersys_set_background::
;; Codigo para la descompresion de backgrounds
      ;ld hl,  #_mimapa_end
      ld de,  #0xFFFF
      call cpct_zx7b_decrunch_s_asm
ret

;rendersys_clear::

;   ld  de, #0xC000 ;;[3] DE = Pointer to start of the screen
;   ld   c, e_x(ix)   ;; x
;   ld   b, e_y(ix)  ;; y

;   call cpct_getScreenPtr_asm ;;Calculate video memory location

;   ex de, hl
;   ld   a, #0x00     ;; color
;   ld   c, e_w(ix)    ;; Width
;   ld   b, e_h(ix)    ;; Height
;  call cpct_drawSolidBox_asm ;; [x] --Esta funcion se carga el valor de las variables que usa

;ret


;;INPUT: IX pointer to the entity
rendersys_clear_entity:: ;;Creo un DrawSolidBox que borra del todo la entidad en pantalla antes de destruirla

   ld    e, e_newVP_l(ix)
   ld    d, e_newVP_h(ix)

   ld    c, e_w(ix)      ;; Width

   ld    b, e_h(ix)      ;; Heith

   xor   a               ;; color = 0
   ;ld    a, #0xFF
   call cpct_drawSolidBox_asm ;; [x] --Esta funcion se carga el valor de las variables que usa
ret



.macro screenPtrBottle _rum_x, _rum_y

   ;;ahora pasamos a pintar el sprite
   ld de, #0xC000 ;; [3] Puntero al inicio de la memoria de video

   ld c, #_rum_x
   ld b, #_rum_y

   call cpct_getScreenPtr_asm ;;Calculate video memory location       ;;First Bottle

   ex    de, hl

.endm

.macro drawBottle
   ld    c, #4
   ld    b, #16

   call cpct_drawSprite_asm
.endm

change_lives_sprites::
   call entityman_getEntityArray_IX



   ld    a, e_hp(ix)          ;; a = lives_player

      cp    #1
      jr    nz, _no_1
   _tiene_1:
      screenPtrBottle _pos_rum1_x, _pos_rum_y
      ld    hl, #_interface_sprites_sp_13
      drawBottle
      
      screenPtrBottle _pos_rum2_x, _pos_rum_y
      ld    hl, #_interface_sprites_sp_13
      drawBottle
      
      screenPtrBottle _pos_rum3_x, _pos_rum_y
      ld    hl, #_interface_sprites_sp_10
      drawBottle


   _no_1:

      cp    #2
      jr    nz, _no_2
   _tiene_2:
      screenPtrBottle _pos_rum1_x, _pos_rum_y
      ld    hl, #_interface_sprites_sp_13
      drawBottle
      
      screenPtrBottle _pos_rum2_x, _pos_rum_y
      ld    hl, #_interface_sprites_sp_10
      drawBottle
      
      screenPtrBottle _pos_rum3_x, _pos_rum_y
      ld    hl, #_interface_sprites_sp_10
      drawBottle
   _no_2:

      cp    #3
      jr    nz, _no_3
   _tiene_3:
      screenPtrBottle _pos_rum1_x, _pos_rum_y
      ld    hl, #_interface_sprites_sp_10
      drawBottle
      
      screenPtrBottle _pos_rum2_x, _pos_rum_y
      ld    hl, #_interface_sprites_sp_10
      drawBottle
      
      screenPtrBottle _pos_rum3_x, _pos_rum_y
      ld    hl, #_interface_sprites_sp_10
      drawBottle
   _no_3:


ret


;;RENDER DE COSAS PARA LA INTERFAZ


;; INPUT: A, num of the level
render_num_level::
   inc   a
   cp    #10
   jr    nc, _dos_digitos
   call  render_num_1_digit
   jr    _fin
   _dos_digitos:
   call  render_num_2_digit
   _fin:
ret

render_num_1_digit::
   ;;calculamos posicion
   ld de, #0xC000 ;; [3] Puntero al inicio de la memoria de video
   ld c, #_pos_num1_x
   ld b, #_pos_num1_y
   push  af
   call cpct_getScreenPtr_asm ;;Calculate video memory location
   pop   af

   ex    de, hl

      cp    #0
      jr    nz, _no_stage_0
   _stage_0:
      ld    hl, #_interface_sprites_sp_00
      jr _dibujar
   _no_stage_0:

   cp    #1
      jr    nz, _no_stage_1
   _stage_1:
      ld    hl, #_interface_sprites_sp_01
      jr _dibujar
   _no_stage_1:

   cp    #2
      jr    nz, _no_stage_2
   _stage_2:
      ld    hl, #_interface_sprites_sp_02
      jr _dibujar
   _no_stage_2:

   cp    #3
      jr    nz, _no_stage_3
   _stage_3:
      ld    hl, #_interface_sprites_sp_03
      jr _dibujar
   _no_stage_3:

   cp    #4
      jr    nz, _no_stage_4
   _stage_4:
      ld    hl, #_interface_sprites_sp_04
      jr _dibujar
   _no_stage_4:

   cp    #5
      jr    nz, _no_stage_5
   _stage_5:
      ld    hl, #_interface_sprites_sp_05
      jr _dibujar
   _no_stage_5:

   cp    #6
      jr    nz, _no_stage_6
   _stage_6:
      ld    hl, #_interface_sprites_sp_06
      jr _dibujar
   _no_stage_6:

   cp    #7
      jr    nz, _no_stage_7
   _stage_7:
      ld    hl, #_interface_sprites_sp_07
      jr _dibujar
   _no_stage_7:

   cp    #8
      jr    nz, _no_stage_8
   _stage_8:
      ld    hl, #_interface_sprites_sp_08
      jr _dibujar
   _no_stage_8:

   cp    #9
      jr    nz, _no_stage_9
   _stage_9:
      ld    hl, #_interface_sprites_sp_09
      jr _dibujar
   _no_stage_9:

   _dibujar:
   ld    c, #4
   ld    b, #16

   call cpct_drawSprite_asm
ret

render_num_2_digit:
   ;; renderizamos el 1 primero
   ld de, #0xC000 ;; [3] Puntero al inicio de la memoria de video
   ld c, #_pos_num1_x-1
   ld b, #_pos_num1_y
   push  af
   call cpct_getScreenPtr_asm ;;Calculate video memory location
   ex    de, hl
   ld    c, #4
   ld    b, #16
   ld    hl, #_interface_sprites_sp_01
   call cpct_drawSprite_asm




   ;; ahora comprobamos el segundo numero
   ;;calculamos posicion
   ld de, #0xC000 ;; [3] Puntero al inicio de la memoria de video
   ld c, #_pos_num1_x+1
   ld b, #_pos_num1_y

   call cpct_getScreenPtr_asm ;;Calculate video memory location
   pop   af

   ex    de, hl

      cp    #10
      jr    nz, _no_stage_10
   _stage_10:
      ld    hl, #_interface_sprites_sp_00
      jr _dibujar_2
   _no_stage_10:

   cp    #11
      jr    nz, _no_stage_11
   _stage_11:
      ld    hl, #_interface_sprites_sp_01
      jr _dibujar_2
   _no_stage_11:

   cp    #12
      jr    nz, _no_stage_12
   _stage_12:
      ld    hl, #_interface_sprites_sp_02
      jr _dibujar_2
   _no_stage_12:

   cp    #13
      jr    nz, _no_stage_13
   _stage_13:
      ld    hl, #_interface_sprites_sp_03
      jr _dibujar_2
   _no_stage_13:

   cp    #14
      jr    nz, _no_stage_14
   _stage_14:
      ld    hl, #_interface_sprites_sp_04
      jr _dibujar_2
   _no_stage_14:

   cp    #15
      jr    nz, _no_stage_15
   _stage_15:
      ld    hl, #_interface_sprites_sp_05
      jr _dibujar_2
   _no_stage_15:

   cp    #16
      jr    nz, _no_stage_16
   _stage_16:
      ld    hl, #_interface_sprites_sp_06
      jr _dibujar_2
   _no_stage_16:

   cp    #17
      jr    nz, _no_stage_17
   _stage_17:
      ld    hl, #_interface_sprites_sp_07
      jr _dibujar_2
   _no_stage_17:

   cp    #18
      jr    nz, _no_stage_18
   _stage_18:
      ld    hl, #_interface_sprites_sp_08
      jr _dibujar_2
   _no_stage_18:

   cp    #19
      jr    nz, _no_stage_19
   _stage_19:
      ld    hl, #_interface_sprites_sp_09
      jr _dibujar_2
   _no_stage_19:

   _dibujar_2:
   ld    c, #4
   ld    b, #16

   call cpct_drawSprite_asm

ret

render_face_sailor::
   ;;;;;;;Restauro el fondo de color rojo
    ;;calculamos posicion
   ld de, #0xC000 ;; [3] Puntero al inicio de la memoria de video
   ld c, #_pos_face_pirate_talk_x
   ld b, #_pos_face_pirate_y
   push  af
   call cpct_getScreenPtr_asm ;;Calculate video memory location
   pop   af
   ex    de, hl

   ld hl, #_pirate_face_quotes_sp_2
   ld    c, #24
   ld    b, #32
   call cpct_drawSprite_asm


      ;;;;;;;Pinto la cara del personaje
    ;;calculamos posicion
   ld de, #0xC000 ;; [3] Puntero al inicio de la memoria de video
   ld c, #_pos_face_pirate_x
   ld b, #_pos_face_pirate_y
   push  af
   call cpct_getScreenPtr_asm ;;Calculate video memory location
   pop   af
   ex    de, hl

   ;;Pinto la cara del personaje
   ld hl, #_pirate_face_big_sp_0
   ld    c, #8
   ld    b, #32
   call cpct_drawSprite_asm

ret



render_update_sailor_face::
    ld  a, (_pirate_state)         
    cp  #0                         
    jp   z, update_sailor_face_END

    update_sailor_face_1:
    ld a, (_pirate_face_animation_cd)
    dec a
    ld (_pirate_face_animation_cd), a

    cp #0
    jr nz, update_sailor_face_END

    ;;AQUI PINTAMOS LA CARA GUAY SI SU cooldown es 0
    call render_face_sailor

    ld a, #0
    ld (_pirate_state), a

    update_sailor_face_END:     
   

ret


hurt_pirate:: ;; Cambiamos el estado del pirata a bien jodido y pintamos

   ;;Pintamos su cara (CARA DEMACRE)
    ;;calculamos posicion
   ld de, #0xC000 ;; [3] Puntero al inicio de la memoria de video
   ld c, #_pos_face_pirate_talk_x
   ld b, #_pos_face_pirate_y
   push  af
   call cpct_getScreenPtr_asm ;;Calculate video memory location
   pop   af
   ex    de, hl

   ld hl, #_pirate_face_quotes_sp_1
   ld    c, #24
   ld    b, #32
   call cpct_drawSprite_asm


   ;;Ponemos a maximo el cooldown
   ld a, #30
   ld (_pirate_face_animation_cd), a

   ;;Cambiamos el estado del pirata
   ld a, #1
   ld (_pirate_state), a

ret


down_you_go::

;;Pintamos su cara (CARA DEMACRE)
    ;;calculamos posicion
   ld de, #0xC000 ;; [3] Puntero al inicio de la memoria de video
   ld c, #_pos_face_pirate_talk_x
   ld b, #_pos_face_pirate_y
   push  af
   call cpct_getScreenPtr_asm ;;Calculate video memory location
   pop   af
   ex    de, hl

   ld hl, #_pirate_face_quotes_sp_0
   ld    c, #24
   ld    b, #32
   call cpct_drawSprite_asm


   ;;Ponemos a maximo el cooldown
   ld a, #30
   ld (_pirate_face_animation_cd), a

   ;;Cambiamos el estado del pirata
   ld a, #1
   ld (_pirate_state), a
ret

;; INPUT
;;    IX, pointer to fort to be rendered
render_fort_IX::
   ;; Calculamos posicion 
   ld de, #0xC000 ;; [3] Puntero al inicio de la memoria de video
   ld c, e_x(ix)
   ld b, e_y(ix)
   call cpct_getScreenPtr_asm ;;Calculate video memory location

   ex de, hl

   ld e_newVP_l(ix), e ;; Guardamos los valores de la nueva posicion
   ld e_newVP_h(ix), d

   ld e_lastVP_l(ix), e ;; Guardamos los valores de la posicion anterior
   ld e_lastVP_h(ix), d


   ld   h, e_sp_h(ix)
   ld   l, e_sp_l(ix)   ;;Etiqueta que referencia al codigo del sprite
   
   ld   c, e_w(ix) ;;Ancho de la entidad
   ld   b, e_h(ix) ;;Alto de la entidad

   call cpct_drawSprite_asm

   ;; ya no se tiene que volver a dibujar
   xor   a
   ld    e_render_fort(ix), a

ret

render_fort_IY::
   ld__a_ixl
   ld__iyl_a

   ld__a_ixh
   ld__iyh_a
   call render_fort_IX
ret


check_render_forts::
   call entityman_getFortArray_IY
   call entityman_getNumForts_A
   cp #0 ;;si no hay fuertes acaba
   ret   z

   ld    b, a     ;;load in B number of forts

   _check_next_fort:
      ;;tenemos que acceder a los datos del fuerte
      ;;meteremos en iy el puntero a la entidad del fuerte
      ld a, ptr_fort_l(iy)
      ld__ixl_a
      ld a, ptr_fort_h(iy)
      ld__ixh_a

      ;;hay que ver si se tiene que renderizar
      ld    a, e_render_fort(ix)
      cp    #0
      jr    z, _next_fort
      push  bc
      call  render_fort_IX
      pop   bc

   _next_fort:
      dec   b        ;; num_forts--
      ret   z      ;; fin if  num_forts = 0

      ld    de, #sizeof_ptr_to_fort    ;; DE = ptr_size
      add   iy, de      ;; IX, next entity

      jr _check_next_fort
ret
